/*****************************************************************************
 *                                                                           *
 *        **********                                                         *
 *       ************                                                        *
 *      ***        ***                                                       *
 *      ***   +++   ***                                                      *
 *      ***   + +   ***                                                      *
 *      ***   +                            CHIPCON CC1010                    *
 *      ***   + +   ***                CHIPCON UTILITY LIBRARY               *
 *      ***   +++   ***                                                      *
 *      ***       ***                                                        *
 *       ***********                                                         *
 *        *********                                                          *
 *                                                                           *
 *****************************************************************************
 * The Chipcon Utility Library is a collection of functions and macros which *
 * implement various basic components that are useful when building RF       *
 * applications on the CC1010.                                               *
 *****************************************************************************
 * Author:              ROH, JOL, OGR                                        *
 *****************************************************************************
 * Revision history:                                                         *
 * 1.0  2002/08/13      First Public Release                                 *
 *                                                                           *
 * $Log: cul.h,v $
 * Revision 1.4  2003/07/29 11:22:14  tos
 * Introduced support for frequency hopping.
 *
 * Revision 1.3  2003/04/03 12:38:59  tos
 * Using sppSetupRF to recalibrate caused the custom callbacks to be reset.
 * Removed timer initialization from sppSetupRF (including the clkFreq argument)
 * sppStartTimer(...) and SPP_INIT_TIMEOUTS() must now be called manually at startup
 *
 * Revision 1.2  2002/11/21 10:51:41  tos
 * Cosmetic change: corrected inconsistent header text.
 *
 * Revision 1.1  2002/10/14 11:49:08  tos
 * Initial version in CVS.
 *
 *                                                                           *
 ****************************************************************************/

#ifndef CUL_H
#define CUL_H

#include <chipcon/reg1010.h>
#include <chipcon/hal.h>




/*****************************************************************************
 *****************************************************************************
 *                                                                           *
 *     00000  00000  00000                                                   *
 *     0      0   0  0                                                       *
 *     0      00000  0              - CYCLIC REDUNDANCY CODE -               *
 *     0      0  0   0                                                       *
 *     00000  0   0  00000                                                   *
 *                                                                           *
 *****************************************************************************
 *****************************************************************************
 * CRC-8 (DOW) and CRC-16 (CCITT) functions that operate on data bytewise    *
 * and in blocks are defined below along with helpful constants.             *
 *                                                                           *
 * Versions:                                                                 *
 *     Fast: Using lookup tables (256 bytes for CRC-8, 512 bytes for CRC-16) *
 *     Small: For-loop (5-6 times slower than the fast implementation)       *
 *****************************************************************************
 * Author:              JOL                                                  *
 ****************************************************************************/




/*****************************************************************************
 *****************************************************************************
 *************              COMMON CRC DEFINITIONS               *************
 *****************************************************************************
 ****************************************************************************/
 
 // Constants used by the CRC algorithms
#define CRC16_POLY 0x1021
#define CRC16_INIT 0xFFFF
#define CRC8_POLY 0x18
#define CRC8_INIT 0xFF

// A constant which can be used to check if a CRC check succeeded, e.g.:
// if (culSmallCRC8(lastDataByte, crcReg) == CRC_OK) ...
#define CRC_OK 0




/*****************************************************************************
 *****************************************************************************
 *************                    FAST CRC-8                     *************
 *****************************************************************************
 ****************************************************************************/
 

// CRC8 LUT (newCRC = crc8LUT[dataByte ^ oldCRC])
extern byte code crc8LUT[256];


//----------------------------------------------------------------------------
//  byte culFastCRC8(crcData, crcReg)
//	Macro FAST_CRC8(crcData, crcReg)
//
//	Description:
//		A CRC-8 (DOW) implementation optimized for fast execution.
//		The function should be called once for each byte in the data
//		the CRC is to be performed on. Before the invocation on the first byte
//		the FAST_CRC8_INIT() macro should be called. This final CRC-value 
//		should be added at the end of the data to facilitate a later CRC  
//		check. During checking the check should be performed on all the data 
//		AND the CRC-8 value appended to it. The data is intact if the value 
//		returned is 0.
//
//	Arguments:
//		byte crcData
//			The data to perform the CRC-8 operation on.
//		byte crcReg
//			The current value of the CRC register. For each additional byte 
//			the value returned for the last invocation should be supplied.
//
//	Return value (not for the macro):
//		The updated value of the CRC8 register. This corresponds to the 
//		CRC-8 of the data supplied so far. During CRC checking, after working
//		through all the data and the appended CRC-8 value, the value will be
//		0 if the data is intact.
//----------------------------------------------------------------------------
byte culFastCRC8(byte crcData, byte crcReg);
#define FAST_CRC8(crcData, crcReg) (crcReg = crc8LUT[crcData ^ crcReg])
#define FAST_CRC8_INIT(crcReg) (crcReg = CRC8_INIT)
#define FAST_CRC8_BLOCK


//----------------------------------------------------------------------------
//  byte culFastCRC8Block(byte *crcData, word length, byte crcReg)
//
//	Description:
//		An implementation of the above function (culFastCRC8) that operates
//      on blocks of data instead of single bytes.
//
//	Extra arguments:
//      word length
//          The number of bytes in this block (crcData[]).
//----------------------------------------------------------------------------
byte culFastCRC8Block(byte *crcData, word length, byte crcReg);
    
    


/*****************************************************************************
 *****************************************************************************
 *************                   SMALL CRC-8                     *************
 *****************************************************************************
 ****************************************************************************/
    
    
//----------------------------------------------------------------------------
//	byte culSmallCRC8(...)
//
//	Description:
//		A CRC-8 (DOW) implementation optimized for small code size.
//		The function should be called once for each byte in the data
//		the CRC is to be performed on. For the invocation on the first byte
//		the value CRC8_INIT should be given for _crcReg_. The value returned
//		is the CRC-8 of the data supplied so far. This CRC-value should be
//		added at the end of the data to facilitate a later CRC check. During
//		checking the check should be performed on all the data AND the CRC-16
//		value appended to it. The data is intact if the value returned is 0.
//
//	Arguments:
//		byte crcData
//			The data to perform the CRC-8 operation on.
//		byte crcReg
//			The current value of the CRC register. For the first byte the
//			value CRC8_INIT should be supplied. For each additional byte the
//			value returned for the last invocation should be supplied.
//
//	Return value:
//		The updated value of the CRC8 register. This corresponds to the 
//		CRC-8 of the data supplied so far. During CRC checking, after working
//		through all the data and the appended CRC-8 value, the value will be
//		0 if the data is intact.
//----------------------------------------------------------------------------
byte culSmallCRC8(byte crcData, byte crcReg);


//----------------------------------------------------------------------------
//  byte culSmallCRC8Block(...)
//
//	Description:
//	    A CRC-8 (DOW) implementation optimized for small code size on blocks
//      of data. For the invocation on the first and/or only block the value
//      CRC8_INIT should be given for _crcReg_. The value returned
//		is the CRC-8 of the data supplied so far. This CRC-value should be
//		added at the end of the data to facilitate a later CRC check. During
//		checking the check should be performed on all the data AND the CRC-8
//		value appended to it. The data is intact if the value returned is 0.
//
//	Arguments:
//		byte* crcData
//			A pointer to the block of data to perform the CRC-8 operation on.
//      word length
//          The number of bytes in this block.
//		byte crcReg
//			The current value of the CRC register. For the first block the
//			value CRC8_INIT should be supplied. For each additional block the
//			value returned for the last invocation should be supplied.
//
//	Return value:
//		The updated value of the CRC8 register. This corresponds to the 
//		CRC-8 of the data supplied so far. During CRC checking, after working
//		through all the data and the appended CRC-8 value, the value will be
//		0 if the data is intact.
//----------------------------------------------------------------------------
byte culSmallCRC8Block(byte* crcData, word length, byte crcReg);




/*****************************************************************************
 *****************************************************************************
 *************                   FAST CRC-16                     *************
 *****************************************************************************
 ****************************************************************************/
 
extern word code crc16LUT[256];

//----------------------------------------------------------------------------
//	word culFastCRC16(crcData, crcReg)
//  Macro FAST_CRC16(crcData, crcReg)
//
//	Description:
//		A CRC-16 (CCITT) implementation optimized for fast execution.
//		The function should be called once for each byte in the data
//		the CRC is to be performed on. Before the invocation on the first byte
//		the FAST_CRC16_INIT() macro should be called. This final CRC-value 
//		should be added at the end of the data to facilitate a later CRC  
//		check. During checking the check should be performed on all the data 
//		AND the CRC-16 value appended to it. The data is intact if the value 
//		returned is 0.
//
//	Arguments:
//		byte crcData
//			The data to perform the CRC-16 operation on.
//		word crcReg
//			The current value of the CRC register. For each additional byte 
//			the value returned for the last invocation should be supplied.
//
//	Return value:
//		The updated value of the CRC16 register. This corresponds to the 
//		CRC-16 of the data supplied so far. During CRC checking, after working
//		through all the data and the appended CRC-16 value, the value will be
//		0 if the data is intact.
//----------------------------------------------------------------------------
word culFastCRC16(byte crcData, word crcReg);
#define FAST_CRC16(crcData, crcReg) (crcReg = (crcReg << 8) ^ crc16LUT[((byte)(crcReg >> 8)) ^ crcData])
#define FAST_CRC16_INIT(crcReg) (crcReg = CRC16_INIT)


//----------------------------------------------------------------------------
//  word culFastCRC16Block(byte *crcData, word length, word crcReg)
//
//	Description:
//		An implementation of the above function (culFastCRC16) that operates
//      on blocks of data instead of single bytes.
//
//	Extra arguments:
//      word length
//          The number of bytes in this block (crcData[]).
//----------------------------------------------------------------------------
word culFastCRC16Block(byte *crcData, word length, word crcReg);




/*****************************************************************************
 *****************************************************************************
 *************                   SMALL CRC-16                    *************
 *****************************************************************************
 ****************************************************************************/


//----------------------------------------------------------------------------
//	word culSmallCRC16(...)
//
//	Description:
//		A CRC-16/CCITT implementation optimized for small code size.
//		The function should be called once for each byte in the data
//		the CRC is to be performed on. For the invocation on the first byte
//		the value CRC16_INIT should be given for _crcReg_. The value returned
//		is the CRC-16 of the data supplied so far. This CRC-value should be
//		added at the end of the data to facilitate a later CRC check. During
//		checking the check should be performed on all the data AND the CRC-16
//		value appended to it. The data is intact if the value returned is 0.
//
//	Arguments:
//		byte crcData
//			The data to perform the CRC-16 operation on.
//		word crcReg
//			The current value of the CRC register. For the first byte the
//			value CRC16_INIT should be supplied. For each additional byte the
//			value returned for the last invocation should be supplied.
//
//	Return value:
//		The updated value of the CRC16 register. This corresponds to the 
//		CRC-16 of the data supplied so far. During CRC checking, after working
//		through all the data and the appended CRC-16 value, the value will be
//		0 if the data is intact.
//----------------------------------------------------------------------------
word culSmallCRC16(byte crcData, word crcReg);


//----------------------------------------------------------------------------
//  word culSmallCRC16Block(...)
//
//	Description:
//	    A CRC-16/CCITT implementation optimized for small code size on blocks
//      of data. For the invocation on the first and/or only block the value
//      CRC16_INIT should be given for _crcReg_. The value returned
//		is the CRC-16 of the data supplied so far. This CRC-value should be
//		added at the end of the data to facilitate a later CRC check. During
//		checking the check should be performed on all the data AND the CRC-16
//		value appended to it. The data is intact if the value returned is 0.
//      It is important that the CRC value is appended to the data in BIG
//      ENDIAN byte order. (Use the CRC16Append(...) function for this.)
//
//	Arguments:
//		byte* crcData
//			A pointer to the block of data to perform the CRC-16 operation on.
//      lword length
//          The number of bytes in this block.
//		word crcReg
//			The current value of the CRC register. For the first block the
//			value CRC16_INIT should be supplied. For each additional block the
//			value returned for the last invocation should be supplied.
//
//	Return value:
//		The updated value of the CRC16 register. This corresponds to the 
//		CRC-16 of the data supplied so far. During CRC checking, after working
//		through all the data and the appended CRC-16 value, the value will be
//		0 if the data is intact.
//----------------------------------------------------------------------------
word culSmallCRC16Block(byte* crcData, word length, word crcReg);


//----------------------------------------------------------------------------
//  void culCRC16Append(...)
//
//	Description:
//	    Appends a CRC value to a block of data in BIG ENDIAN byte order.
//
//	Arguments:
//		byte* dataPtr
//			A pointer to where to insert the CRC value.
//		word crcValue
//			The CRC value to insert.
//
//	Return value:
//      void
//----------------------------------------------------------------------------
void culCRC16Append(byte* dataPtr, word crcValue);


/****************************************************************************/




/*****************************************************************************
 *****************************************************************************
 *                                                                           *
 *     00000  00000  00000                                                   *
 *     0      0   0  0   0                                                   *
 *     00000  00000  00000          - SIMPLE PACKET PROTOCOL -               *
 *         0  0      0                                                       *
 *     00000  0      0                                                       *
 *                                                                           *
 *****************************************************************************
 *****************************************************************************
 * The CUL SPP provides a way to transmit and receive radio packets.         *
 * Higher protocol layers or applications can access the physical layer      *
 * using the simple SPP- interface. The interface consists of functions and  *
 * macros for                                                                *
 *   - Setup of the radio and modem hardware, including RF (re)calibration.  *
 *   - Transmitting and receiving data packets (with timeouts).              *
 *   - Resetting the transceiver (aborting a transmission or reception).     *
 *   - Getting the current transmission/reception status.                    *
 *   - A packet sequence bit which can be used to guarantee that packets are * 
 *     received only once (The bit is toggled after a successfull            *
 *     transmission).                                                        *
 *   - Two free timed callbacks with a precission of 10 msecs.               *
 *   - Getting the current time from a 16-bit 10-msec resolution counter.    *
 *                                                                           *
 * Other features:                                                           *
 * 	 - 255 physical network addresses, and a special broadcast address       *
 *     (SPP_BROADCAST). Packet acknowledging is not supported for broadcasts.*
 * 	 - Error detection through CRC8 (message header) and CRC16 (data)        *
 *   - Automatic retransmissions (variable)                                  *
 *   - Reception timeouts (variable)                                         *
 *                                                                           *
 * How to use the SPP:                                                       *
 *   - Include this file, <chipcon/cul.h>                                    *
 *   - Declare: SPP_SETTINGS, RF_RXTXPAIR_SETTINGS, RF_RXTXPAIR_CALDATA,     *
 *     SPP_RX_INFO and SPP_TX_INFO                                           *
 *   - Initialize SPP_SETTINGS and RF_RXTXPAIR_SETTINGS (Use SmartRF Studio) *
 *   - Call sppSetupRF(...)                                                  *
 *   - For each transmission:                                                *
 *       - Prepare SPP_TX_INFO                                               *
 *       - Call sppSend(...)                                                 *
 *       - Poll the transmission status by using the SPP_STATUS() macro      *
 *   - For each reception:                                                   *
 *       - Prepare SPP_RX_INFO                                               *
 *       - Call sppReceive(...)                                              *
 *       - Poll the reception status by using the SPP_STATUS() macro         *
 *****************************************************************************
 * Author:              JOL                                                  *
 ****************************************************************************/



/*****************************************************************************
 *****************************************************************************
 *************                      Setup                        *************
 *****************************************************************************
 ****************************************************************************/

// Local settings
typedef struct {
	byte myAddress;				// The address of this network node
	word rxTimeout;				// Receive timeout (10s of msecs)
	word txAckTimeout;			// Ack receive timeout (10s of msecs)
	byte txAttempts;			// Transmission attempts (applies only to ack'ed messages)
	byte txPreambleByteCount;	// Number of transmitted preamble bytes
} SPP_SETTINGS;

extern SPP_SETTINGS xdata sppSettings;

// The broadcast address
#define SPP_BROADCAST 0



//----------------------------------------------------------------------------
//  void sppSetupRF (...)
//
//  Description:
//      Set up SPP for transmission or reception.
//		Call this function to (re)calibrate the radio, or to switch between
//		different RF settings.
//
//  Arguments:
//      RF_RXTXPAIR_SETTINGS code* pRF_SETTINGS
//			RF settings (frequencies, modem settings, etc.)
//
//      RF_RXTXPAIR_CALDATA xdata* pRF_CALDATA
//			Calibration results
//
//		bool calibrate
//			Calibrate now. *pRF_CALDATA is written to when calibrate = TRUE,
//			and read from otherwise. Use FALSE if *pRF_CALDATA is valid.
//----------------------------------------------------------------------------
void sppSetupRF (RF_RXTXPAIR_SETTINGS code *pRF_SETTINGS, RF_RXTXPAIR_CALDATA xdata *pRF_CALDATA, bool calibrate);


// RF constants
#define SPP_PREAMBLE_SENSE_BITS	16	 // Number of preamble bits to sense before RX (use min. 16)
#define SPP_ZEROPAD_COUNT 		2	 // Number of trailing zero paddings




/*****************************************************************************
 *****************************************************************************
 *************                  Communication                    *************
 *****************************************************************************
 ****************************************************************************/

// Return values from sppSend() and sppReceive()
#define SPP_BUSY				0
#define SPP_RX_STARTED			1
#define SPP_TX_STARTED			1 

// SPP_TX_INFO/SPP_RX_INFO.flags
#define SPP_ACK_REQ				0x01 // Acknowledge request
#define SPP_ACK					0x02 // Ack flag (used internally)
#define SPP_ENCRYPTED_DATA		0x04 // The packet data is encrypted
#define SPP_SEQUENCE_BIT		0x08 // Toggled when a transmission finishes successfully




//----------------------------------------------------------------------------
// The transmit struct
typedef struct {
	byte destination;				 // The destination of the packet
	byte flags;						 // Flags
	byte dataLen;					 // The length of the transmitted data
	byte xdata *pDataBuffer;		 // The transmitted data
	byte status;					 // The transmission status (result)
} SPP_TX_INFO;

// SPP_TX_INFO.status:
#define SPP_TX_TRANSMITTING		0x01 // The packet is being transmitted
#define SPP_TX_WAITING_FOR_ACK	0x02 // Waiting for the ack packet
#define SPP_TX_ACK_INVALID		0x04 // An invalid ack was received
#define SPP_TX_ACK_TIMEOUT		0x08 // No response
#define SPP_TX_FINISHED			0x10 // OK!
#define SPP_TX_RESET			0x20 // Reset by sppReset()


//----------------------------------------------------------------------------
//  byte sppSend (SPP_TX_INFO xdata *pTXInfo)
//
//  Description:
//      If the transceiver is ready (in idle mode), the transmit section will
//		be powered up and the RF interrupt enabled. The RF ISR will then 
//		transmit the packet (pTXInfo) and receive the ack (if requested). If
//		requested (sppSettings.txAttempts = n), the packet will be re-
//		transmitted (n-1) times, until the ack is received. When finished the 
//		transmit section will be powered down.
//		
//		This function will return immediately and the application can 
//		continue while the ISR transmits the packet. When finished, 
//		sppStatus() will return IDLE_MODE. During the transmission it will 
//		return TX_MODE or TXACK_MODE.
//
//		After the transmission: Use pTXInfo->status to find out what happened:
//			SPP_TX_ACK_INVALID = Something was received, but not the ack
//			SPP_TX_ACK_TIMEOUT = No response
//			SPP_TX_FINISHED
//
//		sppSettings.txAckTimeout gives the ack timeout in msecs.
//
//  Arguments:
//      SPP_TX_INFO xdata *pTXInfo
//			An SPP_TX_INFO struct must be prepared before the transmission,
//			including the following values:
//				destination (SPP_BROADCAST or 1-255)
//				flags (SPP_ACK_REQ | SPP_ENCRYPTED_DATA)
//				dataLen (Length of *pDataBuffer, 0-255)
//				pDataBuffer (pointer to the transmission data buffer)
//
//  Return value:
//		byte
//      	SPP_TX_STARTED if OK
//			SPP_BUSY if not ready
//----------------------------------------------------------------------------
byte sppSend (SPP_TX_INFO xdata *pTXInfo);




//----------------------------------------------------------------------------
// The receive struct:
typedef struct {
	byte source;				 // The packet source address
	byte flags;					 // Flags
	byte maxDataLen;			 // Maximum data length (size of *pDataBuffer)
	byte dataLen;				 // The length of the received data
	byte xdata *pDataBuffer;	 // The received data
	byte status;				 // The reception status (result)
} SPP_RX_INFO;

// SPP_RX_INFO.status:
#define SPP_RX_WAITING	 	0x01 // Waiting for packet
#define SPP_RX_RECEIVING 	0x02 // Receiving packet
#define SPP_RX_ACKING		0x04 // Transmitting ack
#define SPP_RX_TIMEOUT 		0x08 // Reception timed out
#define SPP_RX_TOO_LONG		0x10 // The packet data was longer than the given maximum length (the buffer was not updated)
#define SPP_RX_FINISHED		0x20 // OK!
#define SPP_RX_RESET		0x40 // Reset by sppReset()

//----------------------------------------------------------------------------
//  byte sppReceive (SPP_RX_INFO xdata* pRXInfo)
//
//  Description:
//      If the transceiver is ready (in idle mode), the receive section will
//		be powered up and the RF interrupt enabled. The RF ISR will then 
//		receive the packet and transmit an ack if requested to. When finished,
//		the receive section will be powered down.
//		
//		This function will return immediately and the application can 
//		continue while the ISR receives the packet. When finished, sppStatus() 
//		will return SPP_IDLE_MODE. During the transmission it will 
//		return RX_MODE or RXACK_MODE.
//
//		After the reception: Use pRXInfo->status to find out what happened:
//			SPP_RX_TIMEOUT =   Timeout (nothing received).
//			SPP_RX_TOO_LONG =  dataLen > maxDataLen (the buffer is invalid).
//			SPP_RX_FINISHED	=  source, dataLen and *pDataBuffer in *pRXInfo  
//			                   are valid.
//
//  Arguments:
//      SPP_RX_INFO xdata* pRXInfo
//			An SPP_RX_INFO struct must be prepared before the reception,
//			including the following values:
//				maxDataLen (Length of the data buffer, 0-255)
//				pDataBuffer (pointer to the reception buffer)
//
//  Return value:
//		byte
//			SPP_RX_STARTED if OK
//			SPP_BUSY if not ready
//----------------------------------------------------------------------------
byte sppReceive (SPP_RX_INFO xdata *pRXInfo);




//----------------------------------------------------------------------------
//  byte sppStatus (void)
//
//  Description:
//      Returns the status of the SPP finite state machine.
//
//  Return value:
//		byte
//			SPP_IDLE_MODE =  Ready to transmit or receive
//			SPP_TX_MODE =    Transmitting a packet
//			SPP_TXACK_MODE = Waiting for the ack
//			SPP_RX_MODE =    Waiting for or receiving a packet
//			SPP_RXACK_MODE = Transmitting the ack
//
//	NOTE: This function has also been implemented as a macro, SPP_STATUS(),
//	which is both smaller and faster.
//----------------------------------------------------------------------------
byte sppStatus(void);

// #define SPP_STATUS() ... is located in sppInternal.h

// Return values:
#define SPP_IDLE_MODE	0x00
#define SPP_TX_MODE		0x10
#define SPP_TXACK_MODE	0x20
#define SPP_RX_MODE		0x40
#define SPP_RXACK_MODE	0x80




//----------------------------------------------------------------------------
//  void sppReset (void)
//
//  Description:
//		Stops a transmission or reception by 
//			- turning the transceiver off
//			- entering IDLE mode (sppStatus()) 
//----------------------------------------------------------------------------
void sppReset (void);




/*****************************************************************************
 *****************************************************************************
 *************               Timer with Callback                 *************
 *****************************************************************************
 ****************************************************************************/

// SPP supports 2 custom callbacks which can be set dynamically

//----------------------------------------------------------------------------
//  void sppSetTimerCB (void)
//
//	Description:
//		Add timer callback
//
//	Arguments:
//		byte cb
//			Callback index - use SPP_CUSTOM_0_TIMER	or SPP_CUSTOM_1_TIMER
//			The SPP finite state machine uses SPP_FSM_TIMER
//		void (*pF) ()
//			Pointer to the function to call
//		word *pTicks
//			The timeout in 10s of msecs
//
//	NOTE: *pTicks will be decremented by the Timer 3 interrupt 
//----------------------------------------------------------------------------
void sppSetTimerCB (byte cb, void (*pF) (), word *pTicks);

// Available callbacks 
#define SPP_FSM_TIMER		0 // used by the SPP
#define SPP_CUSTOM_0_TIMER	1 // free
#define SPP_CUSTOM_1_TIMER	2 // free



//----------------------------------------------------------------------------
//  word sppGetTime (void)
//
//	Description:
//		Returns the value of the 10-msec counter, which is started by 
//		sppSetupRF(...)
//
//	Return value:
//		The current time in 10s of msecs
//----------------------------------------------------------------------------
word sppGetTime (void);




//----------------------------------------------------------------------------
//  MACRO: SPP_CHANGE_TICKS(var, value)
//
//	Description:
//		Use this macro to change the tick counter variables used with the 
//		callback timer
//
//	Arguments
//		(word) var
//			The variable
//		(word) value
//			The new value
//----------------------------------------------------------------------------
#define SPP_CHANGE_TICKS(var, value)		\
	do {									\
		INT_ENABLE (INUM_TIMER3, INT_OFF);	\
		var = (word) value;					\
		INT_ENABLE (INUM_TIMER3, INT_ON);	\
	} while (0)




// sppInternal.h requires some of the type definitions listed above!
#include <chipcon/sppInternal.h>

/****************************************************************************/




/*****************************************************************************
 *****************************************************************************
 *                                                                           *
 *     00000  00000   0000                                                   *
 *     0   0    0    0                                                       *
 *     00000    0    0              - REAL TIME CLOCK -                      *
 *     0 0      0    0                                                       *
 *     0   0    0     0000                                                   *
 *                                                                           *
 *****************************************************************************
 *****************************************************************************
 * The RealTime clock keeps track of the time. Its main advantage is that it *
 * works even when the main clock source is changed, though with some        *
 * inaccuracy.                                                               *
 *****************************************************************************
 * Author:              OGR                                                  *
 ****************************************************************************/


//----------------------------------------------------------------------------
//  void RTstart()
//
//  Description:
//      Starts the real time clock.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void RTstart();

//----------------------------------------------------------------------------
//  void RTstop()
//
//  Description:
//      Stops the real time clock.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void RTstop();

//----------------------------------------------------------------------------
//  ulong getTime()
//
//  Description:
//      Reads the current time.
//
//  Arguments:
//      None.
//          
//  Return value:
//      ulong
//          The current time in microseconds.
//----------------------------------------------------------------------------
ulong getTime();

//----------------------------------------------------------------------------
//  void setTime(...)
//
//  Description:
//      Sets the real time clock to the given time.
//
//  Arguments:
//      ulong time
//          The current time in microseconds.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setTime(ulong time);

//----------------------------------------------------------------------------
//  void setModulus(...)
//
//  Description:
//      Specifies that whenever the time reaches the given value,
//      it is reset to zero. A modulus of 0 cancels this effect.
//
//  Arguments:
//      ulong modulus
//          The time will be measured modulo this value.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setModulus(ulong modulus);

//----------------------------------------------------------------------------
//  void useX32()
//
//  Description:
//      Switches to the 32 kHz crystal oscillator as main clock source.
//      The oscillator must be powered up prior to calling this function.
//      It is absolutely necessary to use this function when switching
//      to the 32 kHz oscillator if the real time clock is to work properly.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void useX32();

//----------------------------------------------------------------------------
//  void useXOSC()
//
//  Description:
//      Switches to the main crystal oscillator as main clock source.
//      The oscillator must be powered up prior to calling this function.
//      It is absolutely necessary to use this function when switching
//      to the main oscillator if the real time clock is to work properly.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void useXOSC();


/****************************************************************************/




/*****************************************************************************
 *****************************************************************************
 *                                                                           *
 *     00000  0   0  0000                                                    *
 *     0      0   0  0   0                                                   *
 *     0000   00000  0000           - FREQUENCY HOPPING BASICS -             *
 *     0      0   0  0   0                                                   *
 *     0      0   0  0000                                                    *
 *                                                                           *
 *****************************************************************************
 *****************************************************************************
 * The basic methods for supporting frequency hopping include functions that *
 * calibrate the frequencies and change between them.                        *
 * When using FHP, the functions in FHB should not be called by the user     *
 * application as that can interfere with the operation of FHP.              *
 *****************************************************************************
 * Author:              OGR                                                  *
 ****************************************************************************/

typedef struct {
    byte    freq_2a;            // RX channel frequency (A)
    byte    freq_1a;
    byte    freq_0a;
    byte    freq_2b;            // TX channel frequency (B)
    byte    freq_1b;
    byte    freq_0b;
    byte    pll_rx;             // refdiv
    byte    pll_tx;             // refdiv
} FHP_RXTXPAIR_SETTINGS;

// The following variables must be declared by the user application.
// That is typically done in the file Definitions.c.
// See that file for details.
extern code FHP_RXTXPAIR_SETTINGS rxtxpair_settings[];
extern code RF_RXTXPAIR_SETTINGS RF_SETTINGS;
extern xdata byte numberOfFrequencies;
extern xdata RF_RXTXPAIR_CALDATA rxtxpair_caldata[];
extern xdata byte frequencyPermutation[];

// If the halRFOverrideXXX functions are being used,
// this variable must be set to TRUE.
extern xdata bool usingOverrides;

//----------------------------------------------------------------------------
//  void initializeFrequencies()
//
//  Description:
//      Calibrates all the frequencies that are to be used.
//      This function must be called before transmitting or receiving.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void initializeFrequencies();

//----------------------------------------------------------------------------
//  void changeFreq(...)
//
//  Description:
//      Used to change the current frequency and/or mode.
//
//  Arguments:
//      byte mode
//          One of RF_OFF, RF_RX and RF_TX.
//      byte increment
//          The amount by which the position in the frequency
//          hopping sequence will be advanced.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void changeFreq(byte mode, byte increment);

//----------------------------------------------------------------------------
//  byte getFrequencyPosition()
//
//  Description:
//      Reads the current position in the frequency hopping sequence.
//
//  Arguments:
//      None.
//          
//  Return value:
//      byte
//			The current position.
//----------------------------------------------------------------------------
byte getFrequencyPosition();

//----------------------------------------------------------------------------
//  void setFrequencyPosition(...)
//
//  Description:
//      Sets the current position in the frequency hopping sequence.
//
//  Arguments:
//      byte pos
//          The new position in the sequence.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void setFrequencyPosition(byte pos);


/****************************************************************************/



/*****************************************************************************
 *****************************************************************************
 *                                                                           *
 *     00000  0   0  00000                                                   *
 *     0      0   0  0   0                                                   *
 *     0000   00000  00000          - FREQUENCY HOPPING PROTOCOL -           *
 *     0      0   0  0                                                       *
 *     0      0   0  0                                                       *
 *                                                                           *
 *****************************************************************************
 *****************************************************************************
 * The Frequency Hopping Protocol (FHP) is a complete suite that easily      *
 * allows applications to benefit from the advantages of frequency hopping.  *
 *****************************************************************************
 * Author:              OGR                                                  *
 ****************************************************************************/

// This struct contains the settings used by FHP.
// An instance of this struct named fhpSettings must be declared by the user
// application and specify the preferred settings.
// That is typically done in the file Definitions.c.
typedef struct {
	word period;                // The period in milliseconds between beacons
	byte overhead;              // The approximate time in milliseconds that passes from the chip is interrupted to the beacon is received
	byte slotSize;              // The time in milliseconds a slave uses to transmit a request to send a packet
	byte timeslots;             // The number of slotSize periods the master will listen for requests
	byte listenTime;            // The timeout in tenths of milliseconds when listening for beacons, packets and acknowledgements
	bool useLEDs;               // If TRUE, status information is displayed on the LEDs on the evaluation board.
	bool useAck;                // If TRUE, all non-broadcasted packets are acknowledged.
	word clkFreq;               // The clock frequency of the main crystal oscillator of the chip in kilohertz
} FHP_SETTINGS;

extern xdata FHP_SETTINGS fhpSettings;


#define BROADCASTID 0

// An incoming packet is placed in the buffer message,
// and its length in messageLength.
extern xdata byte messageLength;
extern xdata byte message[];



//----------------------------------------------------------------------------
//  void initializeFHP(...)
//
//  Description:
//      Used to initialize the FHP protocol as either a master or a slave.
//      Frequencies are calibrated and timers are set up.
//
//  Arguments:
//      bool master
//          TRUE if master, FALSE if slave
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void initializeFHP(bool master);


//----------------------------------------------------------------------------
//  void setMode(...)
//
//  Description:
//      Determines how active the FHP will be.
//      If the FHP is set up as master, there are two alternatives:
//       - OFF:     The FHP is turned off and does nothing
//       - ACTIVE:  Beacons are transmitted periodically,
//                  packets from the slaves are received and
//                  any packets sent with newSending are transmitted.
//      If the FHP is set up as slave, there are three alternatives:
//       - OFF:     The FHP is turned off and does nothing
//       - PASSIVE: A timer is running in the background keeping track of
//                  where in the frequency sequence the master is.
//                  Packets sent with newSending are transmitted fast,
//                  but nothing is received (unless possibly when
//                  transmitting a packet itself).
//       - ACTIVE:  The FHP listens for every beacon transmitted by
//                  the master. Thus, the communication is full duplex.
//
//  Arguments:
//      byte mode
//          One of OFF, PASSIVE or ACTIVE
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setMode(byte mode);
#define OFF      0
#define PASSIVE  1
#define ACTIVE   2


//----------------------------------------------------------------------------
//  void setReceivingFunction(...)
//
//  Description:
//      Used to specify which function should be called when
//      a packet arrives. The function may simply set a flag to
//      indicate the arrival, or it may process the packet itself.
//      The packet is available in the buffer message until a new
//      packet arrives.
//      An argument of NULL specifies that no function will be called.
//      This is the default behaviour on start-up.
//
//  Arguments:
//      void (*pF) ()
//          A pointer to the function that will be notified of new packets.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setReceivingFunction( void (*pF) () );

//----------------------------------------------------------------------------
//  void setCallbackFunction(...)
//
//  Description:
//      Used to specify which function should be called when
//      a packet is successfully sent.
//      An argument of NULL specifies that no function will be called.
//      This is the default behaviour on start-up.
//
//  Arguments:
//      void (*pF) ()
//          A pointer to the function that will be notified
//          of successful transmissions.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setCallbackFunction( void (*pF) () );

//----------------------------------------------------------------------------
//  void setSynchronizationFailedFunction(...)
//
//  Description:
//      Used to specify which function should be called when
//      an acquisition attempt is unsuccessful.
//      The user application must then initiate a new attempt.
//      This could be done in a number of ways:
//        - calling the synchronize function
//        - calling setMode(ACTIVE)
//        - sending a new packet if in the PASSIVE mode
//          (any previous packet must be canceled)
//      An argument of NULL specifies that the FHP should go on trying
//      to synchronize until it is successful instead of calling a
//      user function. This is the default behaviour on start-up.
//
//  Arguments:
//      void (*pF) ()
//          A pointer to the function that will be notified failed acquisitions.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setSynchronizationFailedFunction( void (*pF) () );

//----------------------------------------------------------------------------
//  bool newSending(...)
//
//  Description:
//      Initiates a new transmission.
//      The previous transmission must either have been successfully completed
//      or canceled.
//      The function returns immediately, and the packet must not be altered
//      before the packet has been sent or canceled.
//
//  Arguments:
//      byte xdata* packet
//          A pointer to the packet, which must reside in external memory.
//      byte length
//          The length of the packet in bytes.
//      byte destination
//          If the FHP is set up as slave, this argument has no effect as the
//          packet is sent to the master anyway.
//          If set up as master, the argument specifies the ID of the slave
//          to receive the packet or BROADCASTID if all slaves should receive
//          the packet.
//          
//  Return value:
//      bool
//          Returns TRUE if the previous transmission either has been
//          successfully completed or canceled. Otherwise, FALSE is returned
//          and the new packet is discarded.
//----------------------------------------------------------------------------
bool newSending(byte xdata* packet, byte length, byte destination);


//----------------------------------------------------------------------------
//  byte getSendStatus()
//
//  Description:
//      Used to check the status of the last initiated transmission.
//
//  Arguments:
//      None.
//          
//  Return value:
//      byte
//          The return value is one of:
//           - IDLE if the last transmission is canceled or completed
//           - NOLINK if this is a slave and
//             there is no radio link with master
//           - WAITING otherwise
//----------------------------------------------------------------------------
byte getSendStatus();
#define IDLE     0
#define WAITING  1
#define NOLINK   2

//----------------------------------------------------------------------------
//  void cancelSending()
//
//  Description:
//      Cancels the current transmission. Thus, it will not be transmitted
//      and a new transmission may be initiated.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void cancelSending();


//----------------------------------------------------------------------------
//  bool synchronize()
//
//  Description:
//      Performs an acquisition by scanning through the frequency sequence.
//
//  Arguments:
//      None.
//          
//  Return value:
//      bool
//          TRUE if a stable connection was obtained, FALSE otherwise
//----------------------------------------------------------------------------
bool synchronize();

//----------------------------------------------------------------------------
//  void setSystemID(...)
//
//  Description:
//      Sets the system ID. The master and all slaves that are to communicate
//      with each other must have the same system ID. The reason is that
//      all beacons and acknowledgements contain the system ID and are
//      discarded if they do not match. Thus, giving systems different
//      IDs prevents them from interfering with each other.
//      All application programs should choose a system ID different from
//      the default one, preferably chosen randomly for each particular system.
//
//  Arguments:
//      byte ID
//          The new system ID. Range 0-127.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void setSystemID(byte ID);

//----------------------------------------------------------------------------
//  void setOwnID(...)
//
//  Description:
//      If the FHP is set up as master, this function has no effect.
//      Otherwise, it specifies the ID of the slave. The slave will only
//      receive packets addressed to the BROADCASTID or to its own ID.
//      The most significant bit in the ID must be 0, and the ID must not
//      equal the BROADCASTID (0).
//
//  Arguments:
//      byte ID
//          The new ID of the slave. Must be in the range 1-127.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setOwnID(byte ID);


/****************************************************************************/



/*****************************************************************************
 *****************************************************************************
 *                                                                           *
 *     00000  0   0   0000                                                   *
 *     0      0   0  0                                                       *
 *     0000   00000   000          - FREQUENCY HOPPING SPP -                 *
 *     0      0   0      0                                                   *
 *     0      0   0  0000                                                    *
 *                                                                           *
 *****************************************************************************
 *****************************************************************************
 * The Frequency Hopping Simple Packet Protocol (FHSPP) emulates the Simple  *
 * Packet Protocol (SPP) using the Frequency Hopping Protocol (FHP) instead. *
 * Thus, an application developed for SPP can easily be updated to benefit   *
 * from the advantages of frequency hopping.                                 *
 *****************************************************************************
 * Author:              OGR                                                  *
 ****************************************************************************/

// Does nothing.
// initializeFHP must be used instead
#define fhsppSetupRF(a,b,c) 

// emulates sppSend
byte fhsppSend (SPP_TX_INFO xdata *pTXInfo);

// emulates sppReceive
byte fhsppReceive (SPP_RX_INFO xdata *pRXInfo);

// emulates sppStatus
byte fhsppStatus(void);


// emulates sppReset
void fhsppReset (void);


/****************************************************************************/


#endif // CUL_H

