/*****************************************************************************
 *                                                                           *
 *        **********                                                         *
 *       ************                                                        *
 *      ***        ***                                                       *
 *      ***   +++   ***                                                      *
 *      ***   + +   ***                                                      *
 *      ***   +                            CHIPCON CC1010                    *
 *      ***   + +   ***             HARDWARE ABSTRACTION LIBRARY             *
 *      ***   +++   ***                                                      *
 *      ***       ***                                                        *
 *       ***********                                                         *
 *        *********                                                          *
 *                                                                           *
 *****************************************************************************
 * The Chipcon Hardware Abstraction Library is a collection of functions     *
 * which can be used to ease access to the hardware of CC1010, perform       *
 * various calculations, or considered as demonstrations of how to use       *
 * various features of the CC1010.                                           *
 * A number of helpful constants and macros are also included which can      *
 * be used to increase the legibility of your code.                          *
 *****************************************************************************
 * Author:              ROH                                                  *
 *****************************************************************************
 * Revision history:                                                         *
 * 1.1  2002/09/20      Bug fixes in several macros                          *
 * 1.0  2002/04/01      First Public Release                                 *
 *                                                                           *
 * $Log: hal.h,v $
 * Revision 1.9  2004/01/08 18:59:08  tos
 * Bugfix:
 * - Corrected inconsistent macro for frequency synthesizer reset.
 * - Corrected limit algorithm for Timer23 function.
 *
 * Revision 1.8  2003/09/17 14:20:06  tos
 * Bugfix of UART1 makro.
 *
 * Revision 1.7  2003/08/11 15:04:19  tos
 * Corrected reentrant usage in library: reentrant only if using large memory model.
 *
 * Revision 1.6  2003/07/29 11:23:55  tos
 * Made halWait() and halConfigTimer23() reentrant.
 *
 * Revision 1.5  2003/06/26 12:07:25  tos
 * Adjusted/tuned LOCK monitor delay in halRFSetRxTxOff()
 * in accordance with CC1000 datasheet.
 *
 * Revision 1.4  2003/05/12 07:26:25  tos
 * Corrections according to Errata Note 01: recalibrate if unable to lock PLL.
 *
 * Revision 1.3  2003/01/31 09:03:07  tos
 * Corrected UART0_CLOSE macro: ES1=0 should be ES0=0.
 * Ref.: Levent Mehmed (levent_f2000@yahoo.com).
 *
 * Revision 1.2  2002/11/11 08:03:46  tos
 * Checked in for Robin Hoel:
 * Macro for disabling any wait states on XDATA memory accesses.
 * Should always be run at the start of programs.
 *
 * Revision 1.1  2002/10/14 11:49:08  tos
 * Initial version in CVS.
 *
 *                                                                           *
 ****************************************************************************/
#ifndef HAL_H
#define HAL_H                   // Only include this header file once
#include <Chipcon/reg1010.h>    // Include register definitions


/*****************************************************************************
 *****************************************************************************
 *************               Commonly used types                 *************
 *****************************************************************************
 ****************************************************************************/
typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long ulong;

typedef byte bool;


#ifndef TRUE
    #define TRUE 1
#endif
#ifndef FALSE
    #define FALSE 0
#endif
#ifndef NULL
    #define NULL 0
#endif



/*****************************************************************************
 *****************************************************************************
 *************               Port functions/macros               *************
 *****************************************************************************
 ****************************************************************************/

//----------------------------------------------------------------------------
// Macros for more readable access to port data registers / port direction
// registers. Usage examples:
//      PORT(2)=0x3F; tmp=PORT(1);
//      PORTBIT(2,4)=1; PORTBIT(0,1)=PORTBIT(2,5);
//      PORTDIR(2)=PORT_IN(0) | PORT_IN(1) | PORT_OUT(5);   // Out is default
//      PORTDIRBIT(1, 3, POUT);
#define PORT(p)             P##p
#define PORTBIT(p, b)       P ##p## _ ##b
#define PORTDIR(p)          P ##p## DIR
#define PORTDIRBIT(p, b, d) (P ##p## DIR = (d) ? P ##p## DIR | (1<<(b)) : P ##p## DIR & ~(1<<(b)))
#define PORT_OUT(b)         0
#define PORT_IN(b)          (1<<b)
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Functions which allow procedural or indirect access to ports.
        // Sets and gets data bitwise
void halSetPortBit(byte portNum, byte bitNum, byte dataBit);
byte halGetPortBit(byte portNum, byte bitNum);
        // Sets and gets data for entire port
void halSetPort(byte portNum, byte dataByte);
byte halGetPort(byte portNum);
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Functions and macros which allow indirect or procedural access
// to port direction registers.
#define POUT    0
#define PIN     1
        // Sets direction bitwise
void halSetPortBitDir(byte portNum, byte bitNum, byte direction);
        // Sets direction of entire port
void halSetPortDir(byte portNum, byte direction);
//----------------------------------------------------------------------------




/*****************************************************************************
 *****************************************************************************
 *************            Interrupt functions/macros             *************
 *****************************************************************************
 ****************************************************************************/

//----------------------------------------------------------------------------
// Macros which simplify access interrupt enables, interrupt flags and
// interrupt priorities. Increases code legibility a lot.
#define INT_ON                      1
#define INT_OFF                     0
#define INT_HIGH                    1
#define INT_LOW                     0
#define INT_SET                     1
#define INT_CLR                     0

// Global interrupt enables
#define INT_GLOBAL_ENABLE(on)       (EA=(!!on))
#define INT_FLASHDEBUG_ENABLE(on)   (FDIE=(!!on))


// Macro used together with the INUM_* constants defined in Reg1010.h to enable
// or disable certain interrupts. E.g:
//          INT_ENABLE(INUM_RF, INT_ON);
//          INT_ENABLE(INUM_UART0, INT_OFF);
#define INT_ENABLE(inum, on)            \
    do {                                \
        if (inum==INUM_EXTERNAL0)       \
            EX0=(on);                   \
        else if (inum==INUM_EXTERNAL1)  \
            EX1=(on);                   \
        else if (inum==INUM_TIMER0)     \
            ET0=(on);                   \
        else if (inum==INUM_TIMER1)     \
            ET1=(on);                   \
        else if (inum==INUM_TIMER2)     \
            ET2=(on);                   \
        else if (inum==INUM_TIMER3)     \
            ET3=(on);                   \
        else if (inum==INUM_UART0)      \
            ES0=(on);                   \
        else if (inum==INUM_UART1)      \
            ES1=(on);                   \
        else if (inum==INUM_FLASH)      \
            FDIE=(on);                  \
        else if (inum==INUM_RF)         \
            RFIE=(on);                  \
        else if (inum==INUM_DES_ADC)    \
            ADIE=(on);                  \
        else if (inum==INUM_DES)        \
            {if (on) CRPCON|=0x40; else CRPCON&=~0x40;} \
        else if (inum==INUM_ADC)        \
            {if (on) ADCON2|=0x80; else ADCON2&=~0x80;} \
        else if (inum==INUM_RTC)        \
            RTCIE=(on);                 \
    } while (0)


// Macro used together with the INUM_* constants defined in Reg1010.h to set
// the priority of certain interrupts. E.g:
//          INT_PRIORITY(INUM_RF, INT_HIGH);
//          INT_PRIORITY(INUM_UART0, INT_LOW);
#define INT_PRIORITY(inum, p)           \
    do {                                \
        if (inum==INUM_EXTERNAL0)       \
            PX0=(p);                    \
        else if (inum==INUM_EXTERNAL1)  \
            PX1=(p);                    \
        else if (inum==INUM_TIMER0)     \
            PT0=(p);                    \
        else if (inum==INUM_TIMER1)     \
            PT1=(p);                    \
        else if (inum==INUM_TIMER2)     \
            PT2=(p);                    \
        else if (inum==INUM_TIMER3)     \
            PT3=(p);                    \
        else if (inum==INUM_UART0)      \
            PS0=(p);                    \
        else if (inum==INUM_UART1)      \
            PS1=(p);                    \
        else if (inum==INUM_RF)         \
            PRF=(p);                    \
        else if (inum==INUM_DES_ADC)    \
            PAD=(p);                    \
        else if (inum==INUM_RTC)        \
            PRTC=(p);                   \
    } while (0)


// Macro used together with the INUM_* constants defined in Reg1010.h to read
// the interrupt flags:
//          if (INT_GETFLAG(INUM_RF))
//             ...
//          while (!INT_GETFLAG(INUM_UART0_TX));
#define INT_GETFLAG(inum) (             \
        (inum==INUM_EXTERNAL0)  ?       \
            IE0                 :       \
        (inum==INUM_EXTERNAL1)  ?       \
            IE1                 :       \
        (inum==INUM_TIMER0)     ?       \
            TF0                 :       \
        (inum==INUM_TIMER1)     ?       \
            TF1                 :       \
        (inum==INUM_TIMER2)     ?       \
            (!!(EXIF&0x20))     :       \
        (inum==INUM_TIMER3)     ?       \
            (!!(EXIF&0x80))     :       \
        (inum==INUM_UART0_RX)   ?       \
            RI_0                :       \
        (inum==INUM_UART0_TX)   ?       \
            TI_0                :       \
        (inum==INUM_UART1_RX)   ?       \
            RI_1                :       \
        (inum==INUM_UART1_TX)   ?       \
            TI_1                :       \
        (inum==INUM_FLASH)      ?       \
            FDIF                :       \
        (inum==INUM_RF)         ?       \
            (!!(EXIF&0x10))     :       \
        (inum==INUM_DES_ADC)    ?       \
            (!!(EXIF&0x40))     :       \
        (inum==INUM_DES)        ?       \
            (!!(CRPCON&0x20))   :       \
        (inum==INUM_ADC)        ?       \
            (!!(ADCON2&0x40))   :       \
        (inum==INUM_RTC)        ?       \
            RTCIF               :       \
        0                               \
    )


// Macro used together with the INUM_* constants defined in Reg1010.h to set
// or clear certain interrupt flags. E.g:
//          INT_SETFLAG(INUM_RF, INT_SET;
//          INT_SETFLAG(INUM_UART0_RX, INT_CLR);
#define INT_SETFLAG(inum, f)            \
    do {                                \
        if (inum==INUM_EXTERNAL0)       \
            (IE0=(f));                  \
        else if (inum==INUM_EXTERNAL1)  \
            (IE1=(f));                  \
        else if (inum==INUM_TIMER0)     \
            (TF0=(f));                  \
        else if (inum==INUM_TIMER1)     \
            (TF1=(f));                  \
        else if (inum==INUM_TIMER2)     \
            {if (f) EXIF|=0x20; else EXIF&=~0x20;} \
        else if (inum==INUM_TIMER3)     \
            {if (f) EXIF|=0x80; else EXIF&=~0x80;} \
        else if (inum==INUM_UART0_RX)   \
            (RI_0=(f));                 \
        else if (inum==INUM_UART0_TX)   \
            (TI_0=(f));                 \
        else if (inum==INUM_UART1_RX)   \
            (RI_1=(f));                 \
        else if (inum==INUM_UART1_TX)   \
            (TI_1=(f));                 \
        else if (inum==INUM_FLASH)      \
            (FDIF=(f));                 \
        else if (inum==INUM_RF)         \
            {if (f) EXIF|=0x10; else EXIF&=~0x10;} \
        else if (inum==INUM_DES_ADC)    \
            {if (f) EXIF|=0x40; else EXIF&=~0x40;} \
        else if (inum==INUM_DES)        \
            {if (f) CRPCON|=0x20; else CRPCON&=~0x20;} \
        else if (inum==INUM_ADC)        \
            {if (f) ADCON2|=0x40; else ADCON2&=~0x40;} \
        else if (inum==INUM_RTC)        \
            (RTCIF=(f));                \
    } while (0)


// Macros used to set edge/level trigging of the external interrupts.
#define INT_EXTERNAL0_TRIGGER_ON_EDGE()     (IT0=1)
#define INT_EXTERNAL0_TRIGGER_ON_LEVEL()    (IT0=0)
#define INT_EXTERNAL1_TRIGGER_ON_EDGE()     (IT1=1)
#define INT_EXTERNAL1_TRIGGER_ON_LEVEL()    (IT1=0)
//----------------------------------------------------------------------------




/*****************************************************************************
 *****************************************************************************
 *************           Serial port functions/macros            *************
 *****************************************************************************
 *****************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// A number of macros have been defined which allow easy use of the serial
// ports in the CC1010:
//   UART0_SETUP(...)           - Used to configure the serial ports (baudrate,
//   UART1_SETUP(...)             8-bit / 8-bit + parity, interrupt serviced or
//                                polled, RX/TX or just TX.) These functions
//                                also configure pins and interrupts properly.
//   UART0_CLOSE()              - Used to close a serial port after use. Turns
//   UART1_CLOSE()                off baud generation, reverts pins to inputs
//                                and disables relevant interrupts.
//   UART0_SEND(...)            - Used to send a single byte with the relevant
//   UART0_SEND(...)              UART in 8-bit mode. No check is made to see 
//                                if the transmit buffer is empty.
//   UART0_RECEIVE(...)         - Used to receive a single byte with the
//   UART1_RECEIVE(...)           relevant UART in 8-bit mode. No check is made
//                                whether a new byte has been received.
//   UART0_SEND_WITH_PARITY(x)  - Used to send a single byte in the 8-bit + 
//   UART1_SEND_WITH_PARITY(x)    parity mode. No check is made to see if the
//                                transmit buffer is empty.
//   UART0_PARITY_OK()          - Checks to see if the parity of the most 
//   UART1_PARITY_OK()            recently received byte is OK. Returns TRUE or
//                                FALSE.
//   UART0_WAIT_AND_SEND(x)     - Waits until the transmit buffer has been 
//   UART1_WAIT_AND_SEND(x)       emptied (and thus sent) and then sends x 
//                                using the 8-bit mode.
//   UART0_WAIT_AND_RECEIVE(x)  - Waits until a byte is received and then
//   UART1_WAIT_AND_RECEIVE(x)    stores it in x. The 8-bit mode is used.
//
// In addition the functions printf() / scanf() can be used in conjunction
// with UART0 as long as UART0_SETUP(...) is passed the parameter UART_POLLED.
// The printf() / scanf() functions use the putchar() and _getkey() functions
// in libc for actual input and output.
///////////////////////////////////////////////////////////////////////////////
                                  
// Use counter for timer1
extern byte halTimer1Usage;

//----------------------------------------------------------------------------
//  UART0_SETUP(baudRate, clkFreq, options) / UART1_SETUP(...)
//
//  Description:
//      Macro which does all the initialization necessary to establish a 
//      simple serial link on serial port 0/1. Timer1 is used as a baudrate
//      generator and is initialized according to _baudRate_ and _clkFreq_. 
//      The ports required for serial communication (UART0: P3.0 for RX, P3.1 
//      for TX; UART1: P2.0 for RX, P2.1 for TX) are configured as necessary. 
//      The UART is configured according to _options_, the different values 
//      of which are listed below.
//      The first byte is transmitted simply by using the UART0_SEND(...)/
//      UART1_SEND(...) macros.
//
//  Arguments:
//      word baudRate
//          Baudrate (9600, 19200, 57600, ...)
//      word clkFreq
//          Device clock frequency in kHz
//      byte options
//          One or more of the below constants. The value 0 gives the common 
//          setting UART_NO_PARITY | UART_TX_ONLY | UART_ISR.
//----------------------------------------------------------------------------
// options is one or more of these constants:
#define UART_NO_PARITY      0       // 8 data bits, no parity
#define UART_9TH_BIT_PARITY 0x40    // 8 data bits, ninth bit parity
#define UART_RX_TX          0x10    // Receive and transmit
#define UART_TX_ONLY        0       // Transmit only
#define UART_ISR            0       // Use interrupt service routines to 
                                    // signal that a byte has arrived or
                                    // that the TX buffer is empty.
                                    // The user must manually clear RI_0 and
                                    // TI_0 in the corresponding ISRs.
#define UART_POLLED         0x02    // User must poll the RI_0 and TI_0 flags
                                    // to determine when a byte arrives or when
                                    // the TX buffer is empty. The user must
                                    // manually clear RI_0 and TI_0 if not using
                                    // the printf() / scanf() functions which do
                                    // this automatically.

#define UART0_SETUP(baudRate, clkFreq, options)         \
    do {                                                \
        /* Setup pins for UART0 */                      \
        if (!((options)&UART_RX_TX))                    \
            PORTDIRBIT(3, 0, PIN);                      \
        PORTDIRBIT(3, 1, POUT);                         \
        PORTBIT(3, 1)=1;                                \
                                                        \
        /* Setup baudrate & timer 1 */                  \
        CKCON|=0x10;                /* T1M=1 */         \
        PCON|=0x80;                 /* SMOD0=1 */       \
        TH1=256 - ((((ulong)(clkFreq)*10)/((ulong)(baudRate)/100*32) +1)/2);  \
        TR1=1;                                          \
        TMOD=(TMOD&0x0F)|0x20;                          \
        halTimer1Usage++;                               \
                                                        \
        /* Setup SCON0 & interrupts */                  \
        SCON0=(options)+0x40;                           \
        if (!((options)&UART_POLLED)) {                 \
            ES0=1;                                      \
            EA=1;                                       \
        }                                               \
    } while(0)
                                                        
#define UART1_SETUP(baudRate, clkFreq, options)         \
    do {                                                \
        /* Setup pins for UART1 */                      \
        if (!((options)&UART_RX_TX))                    \
            PORTDIRBIT(2, 0, PIN);                      \
        PORTDIRBIT(2, 1, POUT);                         \
        PORTBIT(2, 1)=1;                                \
                                                        \
        /* Setup baudrate & timer 1 */                  \
        CKCON|=0x10;                /* T1M=1 */         \
        EICON|=0x80;                /* SMOD1=1 */       \
        TH1=256 - ((((ulong)(clkFreq)*10)/((ulong)(baudRate)/100*32) +1)/2);  \
        TR1=1;                                          \
        TMOD=(TMOD&0x0F)|0x20;                          \
        halTimer1Usage++;                               \
                                                        \
        /* Setup SCON1 & interrupts */                  \
        SCON1=(options)+0x40;                           \
        if (!((options)&UART_POLLED)) {                 \
            ES1=1;                                      \
            EA=1;                                       \
        }                                               \
    } while(0)
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//  void UART0_CLOSE()/UART1_CLOSE()
//
//  Description:
//      Macro which closes the serial link established by the
//      HAL_SETUP_SERIAL0(...)/HAL_SETUP_SERIAL1(...) macros and performs all 
//      the necessary cleanup: disabling serial interrupts, reverting the pins 
//      used for RX and TX to inputs and turning off timer1 baudgeneration (if 
//      both UART0 and UART1 are turned off.)
//
//  Arguments:
//      None.
//----------------------------------------------------------------------------
#define UART0_CLOSE()                                   \ 
    do {                                                \
        /* Restore pins for UART0 */                    \
        PORTDIRBIT(3, 1, PIN);                          \
                                                        \
        /* Turn off baud generation */                  \
        if (!(--halTimer1Usage))                        \
            TR1=0;                                      \
                                                        \
        /* Turn off interrupts */                       \
        ES0=0;                                          \
    } while (0)

#define UART1_CLOSE()                                   \ 
    do {                                                \
        /* Restore pins for UART1 */                    \
        PORTDIRBIT(2, 1, PIN);                          \
                                                        \
        /* Turn off baud generation */                  \
        if (!(--halTimer1Usage))                        \
            TR1=0;                                      \
                                                        \
        /* Turn off interrupts */                       \
        ES1=0;                                          \
    } while (0)


//-----------------------------------------------------------------------------
// Macros which are helful when transmitting data over a serial interface.
// Usage example:
//      UART0_SEND(data);
//      UART1_SEND_WITH_PARITY(data);
//      for (i=0; i<len; i++)
//          UART0_WAIT_AND_SEND(data[i]);
#define UART0_SEND(x) do { SBUF0=(x); } while (0)
#define UART1_SEND(x) do { SBUF1=(x); } while (0)
#define UART0_SEND_WITH_PARITY(x) do { ACC=(x); TB8_0=P; SBUF0=(x); } while (0)
#define UART1_SEND_WITH_PARITY(x) do { ACC=(x); TB8_1=P; SBUF1=(x); } while (0)
#define UART0_WAIT_AND_SEND(x) do { while (!TI_0); TI_0=0; SBUF0=(x); } while (0)
#define UART1_WAIT_AND_SEND(x) do { while (!TI_1); TI_1=0; SBUF1=(x); } while (0)
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Macros which are helful when receiving data over a serial interface.
// Usage example:
//      len=UART0_RECEIVE();
//      while (len-- > 0) {
//          UART0_WAIT_AND_RECEIVE(data[i++]);
//          if (!UART0_PARITY_OK())
//              break;
//      }
#define UART0_RECEIVE() SBUF0
#define UART1_RECEIVE() SBUF1
#define UART0_PARITY_OK() (ACC=SBUF0, RB8_0==P)
#define UART1_PARITY_OK() (ACC=SBUF1, RB8_1==P)
#define UART0_WAIT_AND_RECEIVE(x) do { while (!RI_0); RI_0=0; (x)=SBUF0; } while (0)
#define UART1_WAIT_AND_RECEIVE(x) do { while (!RI_1); RI_1=0; (x)=SBUF1; } while (0)
//-----------------------------------------------------------------------------



/*****************************************************************************
 *****************************************************************************
 *************          SPI Master functions/macros              *************
 *****************************************************************************
 ****************************************************************************/

//----------------------------------------------------------------------------
// defines which make SPCR (SPI control register) assignments more legible
#define SPI_SPM_ENABLED     0x40    // Enable or disable special purpose
#define SPI_SPM_DISABLED    0x00    // mode (as opposed to regular SPI)

#define SPI_ENABLE          0x20    // Enable SPI
#define SPI_DISABLE         0x00    // Disable SPI.

#define SPI_MSB_FIRST       0x10    // Transmit data MSB first
#define SPI_LSB_FIRST       0x00    // Transmit data LSB first

#define SPI_NEGEDGE_IDLE0   0x04    // 0 when idle, sample on neg. edge, shift on pos. edge
#define SPI_POSEDGE_IDLE0   0x00    // 0 when idle, sample on pos. edge, shift on neg. edge
#define SPI_NEGEDGE_IDLE1   0x08    // 1 when idle, sample on neg. edge, shift on pos. edge
#define SPI_POSEDGE_IDLE1   0x0c    // 1 when idle, sample on pos. edge, shift on neg. edge

#define SPI_CLK_DIV_8       0x00    // SCK frequency = XOSC frequency / 8
#define SPI_CLK_DIV_16      0x01    // SCK frequency = XOSC frequency / 16
#define SPI_CLK_DIV_32      0x02    // SCK frequency = XOSC frequency / 32
#define SPI_CLK_DIV_64      0x03    // SCK frequency = XOSC frequency / 64
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Macro for SPI control (use the constants defined above):
// Example of usage:
//      SPI_CONTROL( SPI_ENABLE | SPI_MSB_FIRST | SPI_POSEDGE_IDLE0 | SPI_CLK_DIV_8 );
//----------------------------------------------------------------------------
#define SPI_CONTROL(x) (SPCR = (x))

//----------------------------------------------------------------------------
// Macro for controlling output of MOSI pin. Should be set as MOSI_OE(TRUE)
// for normal three-wire SPI interfaces. For use with a two-wire SPI interface
// with a bidirectional data line MOSI_OE must be toggled when the direction
// of the data line should be changed.
// Usage example:
//      SPI_CONTROL( SPI_ENABLE | SPI_MSB_FIRST | SPI_POSEDGE_IDLE0 | SPI_CLK_DIV_8 );
//      MOSI_OE(TRUE);               // Master to slave first
//      for (i=0; i<4; i++) {        // Send four bytes
//          SPI_DATA=data_out[i]; while (SPI_IS_ACTIVE);
//      }
//      MOSI_OE(FALSE);              // Slave to master
//      for (i=0; i<4; i++) {        // Receive four bytes
//          SPI_DATA=0;              // Write bogus byte to produce SCLK
//          while (SPI_IS_ACTIVE);
//          data_in[i]=SPI_DATA;
//      }

//----------------------------------------------------------------------------
#define MOSI_OE(x) (P0DIR=(x) ? (P0DIR&~0x02) : (P0DIR|0x02))

//----------------------------------------------------------------------------
// Macro for reading and writing to the SPI data register. Usage example:
//      data_in=SPI_DATA;
//      SPI_DATA=data_out;
//----------------------------------------------------------------------------
#define SPI_DATA SPDR

//----------------------------------------------------------------------------
// Macro used for testing if the SPI interface is in the middle of receiving/
// transmitting a byte at then current time. For maximum transmission
// speeds in a block transfer use the macro SPI_READ_WRITE in a loop.
// Usage example:
//      for (i=0; i<4; i++) {
//          SPI_DATA=spi_out_buf[i];
//          while (SPI_IS_ACTIVE);
//          SPI_DATA=spi_in_buf[i]
//      }
//----------------------------------------------------------------------------
#define SPI_IS_ACTIVE (!!(SPSR&0x02))

//----------------------------------------------------------------------------
// Macro for performing an SPI read/write operation in the fastest possible way.
// Useful when reading/writing a block of data. _din_ receives the incoming data
// byte and _dout_ should contain the data byte to transmit. _din_ and _dout_ 
// can point to the same address as data is transmitted before it is received.
// Usage example:
//      for (i=0; i<4; i++)
//          SPI_READ_WRITE(spi_in_buf[i], spi_out_buf[i]);
//----------------------------------------------------------------------------
#define SPI_READ_WRITE(din, dout)                       \
    do {                                                \
        do SPDR=(dout); while(SPSR&0x01); while(SPI_IS_ACTIVE); (din)=SPDR;   \
    } while(0)

//----------------------------------------------------------------------------
//  void halSpiTransferBlock(...)
//
//  Description:
//      Transfers _length_ bytes of the data block pointed to by _inoutBuffer_
//      over the SPI interface. It is assumed that the SPI interface has
//      already been correctly configured. If _read_ is TRUE the incoming
//      data on the SPI interface will be stored in place in _inoutBuffer_,
//      (overwriting the data that is transmitted), otherwise the received
//      data is ignored.
//
//  Arguments:
//      byte* inoutBuffer
//          Pointer to a block of data that is to be transmitted. If _read_ is
//          TRUE the received data will overwrite this data.
//      word length
//          The number of bytes to receive/transmit.
//      bool enableRead
//          Overwrite the data transmitted with the received data (TRUE) or
//          ignore received data (FALSE).
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halSpiTransferBlock(byte* inoutBuffer, word length, bool enableRead);



/*****************************************************************************
 *****************************************************************************
 *************           FLASH programming functions             *************
 *****************************************************************************
 ****************************************************************************/

//----------------------------------------------------------------------------
//  bool halFlashWritePage(...)
//
//  Description:
//      Writes into the flash page pointed to by _flashPage_ 128 bytes of
//      data pointed to by _ramBuffer_. The addresses of both _flashPage_ and
//      _ramBuffer_ must be an integer multiple of 128, i.e  adr mod 128=0.
//      _clkFreq_ must be the XOSC frequency in kHz and is used to calculate
//      the correct erasure and programming times.
//      If these conditions are not met the function does not perform any
//      programming and returns FALSE. A verification of the data written is
//      performed after programming - if this verification fails for some reason
//      (flash page is write locked, or flash programming failure) the function
//      also returns FALSE.
//      All interrupts are turned off while this function executes.
//
//  Arguments:
//      byte code* flashPage
//          Pointer to the destination of the write in code memory space.
//          Address must be an integer multiple of 128 bytes.
//      byte xdata* ramBuffer
//          Pointer to the data source in RAM for the write in XDATA memory
//          space. Address must be an integer multiple of 128 bytes.
//      word clkFreq
//          The XOSC clock frequency in kHz.
//
//  Return value:
//      bool
//          TRUE if the write was successful. False if programming/verification
//          failed or supplied arguments were invalid.
//----------------------------------------------------------------------------
bool halFlashWritePage(byte code* flashPage, byte xdata* ramBuffer, word clkFreq);


//----------------------------------------------------------------------------
//  bool halCopy2Flash(...)
//
//  Description:
//      Copies _length_ bytes from the memory area pointed to by _src_ (in any
//      memory space) to the memory area in CODE memory space pointed to by
//      _flashPtr_. A 128 byte temporary buffer in the XDATA memory space must
//      be pointed to by _ramBuffer_. None of the pointers (_flashPtr_, _src_
//      or _ramBuffer_) need to be page-aligned (address mod 128=0) as with
//      halFlashWritePage(...), neither does _length_ have to be a multiple of
//      128 bytes -- it can in fact be between 1 and 32768 bytes, although it
//      goes without saying that it is unwise to overwrite the memory occupied
//      by the code for this function. The result is undefined if the memory
//      areas occupied by [_src_, _src_+_length_-1] and [_ramBuffer_,
//      _ramBuffer+127] overlap.
//      A verification of the data written is performed after programming - if
//      this verification fails for some reason (flash page is write locked,
//      flash programming failure) or some of the supplied arguments are
//      invalid the function returns FALSE.
//      All interrupts are turned off while this function executes.
//
//  Arguments:
//      byte code* flashPtr
//          Pointer to the destination of the write in code memory space.
//      byte* src
//          Pointer to the data source.
//      word length
//          The number of bytes to copy.
//      byte xdata* ramBuffer
//          A pointer to a 128 byte big buffer in XDATA memory space used to
//          hold temporary data.
//      word clkFreq
//          The XOSC clock frequency in kHz.
//
//  Return value:
//      bool
//          TRUE if the write was successful. False if programming/verification
//          failed or the supplied arguments were invalid.
//----------------------------------------------------------------------------
bool halCopy2Flash(byte code* flashPtr, byte* src, word length, byte xdata* ramBuffer, word clkFreq);


//---------------------------------------------------------------------------
// Macro used to disable any wait states on XDATA memory accesses. Should
// always be run at the start of programs.
#define MEM_NO_WAIT_STATES() (CKCON&=~0x07)
//---------------------------------------------------------------------------
	
//---------------------------------------------------------------------------
// Macro used to set the FLASH module's power mode. _pm_ must be one of the
// constants defined below. This macro is used to define
#define FLASH_SET_POWER_MODE(pm) (FLCON= (FLCON&~0x60)|(pm&0x60))

#define FLASH_ALWAYS_ON                     0x00
#define FLASH_STANDBY_DURING_IDLE_AND_STOP  0x20
#define FLASH_STANDBY_BETWEEN_READS         0x40
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Macro used to start a manually configured flash write. The macro is
// compatible with the CC1010 serial debugger. Usage example:
//      Setup write parameters...
//      FLASH_WRITE();
//      Continue program...
#define FLASH_WRITE() do {                       \
    CY=EA;                                       \
    EA=0;                                        \
    FLCON |= 0x10;                               \
    RESERVED |= 0x80;                            \
    PCON|=0x01;                                  \
    EA=CY;                                       \
} while(0)
//----------------------------------------------------------------------------



/*****************************************************************************
 *****************************************************************************
 *************            Power and clock management             *************
 *****************************************************************************
 ****************************************************************************/

//----------------------------------------------------------------------------
// Macro used to put the CC1010 in the low-power idle mode. (Processor stops,
// all peripherals run as normal.) The processor continues execution in an ISR
// as soon as an interrupt request comes from an interrupt source which is
// enabled.
#define ENTER_IDLE_MODE() (PCON|=0x01)
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Macro used to put the CC1010 in the very-low-power stop mode. (processor and
// peripherals stopped, main clock tree is gated. Only the ADC and its threshold
// logic is active.) A reset is necessary to continue execution (from the start
// of program memory.) Unless the ADC is enabled and using the main oscillator as
// clock source, the main clock oscillator should be disabled by the
// halMainOscillatorEnable(...) function.
#define ENTER_SLEEP_MODE() do {                         \
    FLASH_SET_POWER_MODE(FLASH_STANDBY_BETWEEN_READS);  \
    PCON|=0x02;                                         \
} while (0)
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Macro used for switching between clock sources for the main clock tree.
#define MAIN_CLOCK_SET_SOURCE(cs) ( X32CON=(X32CON&~0x01)|(cs) )
//The clock source must be one of the below defined constants.
#define CLOCK_XOSC   0
#define CLOCK_X32    1
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Power up/down main crystal oscillator (XOSC)
#define XOSC_ENABLE(bool) ( RFMAIN=(bool) ? (RFMAIN&~0x06) : (RFMAIN|0x06))
// Power up/down 32 kHz crystal oscillator (X32)
#define X32_ENABLE(bool) ( X32CON=(bool) ? (X32CON&~0x02) : (X32CON|0x02) )
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Macro used to configure whether the X32 oscillator is connected to a
// crystal or whether a CMOS-compatible clock signal is applied to the
// XOSC32_Q1 pin.
#define X32_INPUT_SOURCE(s) (X32CON=(X32CON&~0x04) | (s) )
// One of the below constants must be used:
#define X32_USING_CRYSTAL 0
#define X32_USING_EXTERNAL_CLOCK 4
//----------------------------------------------------------------------------



/*****************************************************************************
 *****************************************************************************
 *************              Timer macros/functions               *************
 *****************************************************************************
 ****************************************************************************/

//////////////////////////////////////////////////////////////////////////////
// Timers/Counters 0 & 1 can be configured in a number of ways. The following
// functions allow simple configuration of these in their most common uses:
// as interrupt timers and interrupt pulse counters. Other, more advanced uses
// require manual configuration of the timers/counters.
//////////////////////////////////////////////////////////////////////////////


//----------------------------------------------------------------------------
// ulong halConfigTimer01(...);
//
//  Description:
//      This function configures timer 0 or 1 (depending on the value given in
//      _option_ as either an interrupt timer (an interrupt is generated at
//      certain intervals in time, as specified by _period) or an interrupt
//      pulse counter (an interrupt when _period_ number of pulses have
//      been detected on P3.4/P3.5 for timer0/timer1.) If _period_ is
//      specified as 0, then, in timer mode the timeout period will be set to
//      the maximum possible, and in counter mode the pulse counter will allow
//      the maximum number of pulses before generating an interrupt.
//      Some timer settings (with long timeouts) require that the user 
//      initializes timer register in the interrupt service routine (ISR.) This
//      should be done by using the appropriate version of the 
//      ISR_TIMERx_ADJUST(m) macro with the word pointed to by _modulo_ as an
//      argument. The _modulo_ argument takes a pointer to a word, if it is 
//      NULL, many timer settings will be unavailable. It is the responsibility
//      of the programmer to make sure that the appropriate timer ISR has been
//      declared (and that it begins with the obligatory ISR_TIMERx_ADJUST(m) 
//      macro). The timer must be started with macro TIMERx_RUN(TRUE).
//
//  Arguments:
//      byte options
//          Options indicating which timer to configure and how. Different
//          constants for _option_ is defined below.
//      ulong period
//          The desired period between interrupts in microseconds in timer
//          mode, or the number of counted pulses between interrupts in counter
//          mode.
//          If specified as 0, then, in timer mode the timeout period will be
//          set to the maximum possible, and in counter mode the pulse counter
//          will allow the maximum number of pulses before generating an
//          interrupt. 
//      word clkFreq
//          The XOSC clock frequency in kHz.
//      word xdata* modulo
//          A pointer to a word (in xdata) which after the function has returned
//          contains the value to supply to the obligatory macro invocation of
//          ISR_TIMERx_ADJUST(m) at the start of the timer ISR.
//
//  Return value:
//      ulong
//          In timer mode, the actual period in microseconds between interrupts
//          or zero if the period is impossible to achieve.
//          In counter mode, zero if the supplied count value is impossible to
//          achieve, otherwise one.
//----------------------------------------------------------------------------
ulong halConfigTimer01(byte options, ulong period, word clkFreq, word* modulo);

// option is one or more of:
#define TIMER0                      0   // Configure timer 0
#define TIMER1                      1   // Configure timer 1
#define TIMER01_INT_TIMER           0   // Configure as an interrupt timer
#define TIMER01_INT_PULSE_COUNTER   2   // Configure as an interrupt pulse counter
#define TIMER01_NO_INT_TIMER        4   // Configure as timer that doesn't 
                                        // generate interrupts


// Adjust macro used for timer adjustment in timer 0/1 ISRs.
// This macro should be used at the start of the ISR of timer 0.
#define ISR_TIMER0_ADJUST(m) {  \
    if (m) {                    \
        TR0=0;                  \
        TL0+=(m)&0xFF;          \
        TH0+=(m)>>8;            \
        TR0=1;                  \
    }                           \
}

// Adjust macro used for timer adjustment in timer 0/1 ISRs.
// This macro should be used at the start of the ISR of timer 1.
#define ISR_TIMER1_ADJUST(m) {  \
    if (m) {                    \
        TR1=0;                  \
        TL1+=(m)&0xFF;          \
        TH1+=(m)>>8;            \
        TR1=1;                  \
    }                           \
}
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// ulong halConfigTimer23(...);
//
//  Description:
//      This function configures timer 2 or 3 (depending on the value given in
//      _option_ as either an interrupt timer (an interrupt is generated at
//      certain intervals in time, as specified by _period) or a pulse width 
//      modulator (PWM).
//      If _period_ is specified as 0, then, in timer mode the timeout period will
//      be set to the maximum possible, and in PWM mode the period will be
//      set as long as possible. Using the PWM mode of timer 2/3 overrides the
//      normal operation of ports P3.4/P3.5 and can thus not be used in conjunction
//      with timer 0/1 configured as counters. The duty cycle is set to 50% (128/255)
//      initially in PWM mode.
//      The timer/PWM must be started with macro TIMERx_RUN(TRUE).
//
//  Arguments:
//      byte options
//          Options indicating which timer to configure and how. Different
//          constants for _option_ is defined below.
//      ulong period
//          The desired period between interrupts in microseconds. In PWM mode
//          the duty cycle will be set as close to 50% as possible. This duty
//          cycle can be changed (in an ISR or at any other time) by using the
//          appropriate PWMx_SET_DUTY_CYCLE(...) macro. If _period_
//          is 0, then the maximum period possible will be set. The period can
//          also be adjusted dynamically with the PWMx_SET_PERIOD(...) macro.
//      word clkFreq
//          The XOSC clock frequency in kHz.
//
//  Return value:
//      ulong
//          The actual period in microseconds or zero if the desired period is
//          too high.
//----------------------------------------------------------------------------
#if (__MODEL__ == 0)  // Memory model: 0 = SMALL, 1 = COMPACT, 2 = LARGE
ulong halConfigTimer23(byte options, ulong period, word clkFreq) ;
#else
ulong halConfigTimer23(byte options, ulong period, word clkFreq) reentrant;
#endif

// option is one or more of:
#define TIMER2                      0   // Configure timer 2
#define TIMER3                      1   // Configure timer 3
#define TIMER23_INT_TIMER           4   // Configure as an interrupt timer
#define TIMER23_PWM                 2   // Configure as PWM
#define TIMER23_NO_INT_TIMER        0   // Configure as a timer that doesn't 
                                        // generate interrupts
//----------------------------------------------------------------------------

// Macro for adjusting the duty cycle of PWMs 2/3 in increments of 1/255
#define PWM2_SET_DUTY_CYCLE(cycle) \
    ( T2= (cycle) )
#define PWM3_SET_DUTY_CYCLE(cycle) \
    ( T3= (cycle) )

// Macro for adjusting the period of PWMs 2/3 in increments of CLK/255. 
#define PWM2_SET_PERIOD(period) \
    ( T2PRE=(period) )
#define PWM3_SET_PERIOD(period) \
    ( T3PRE=(period) )


//----------------------------------------------------------------------------
// void halConfigRealtimeClock(...);
//
//  Description:
//      This function configures the realtime clock. It's period in seconds
//      is given in _period_. In order for the realtime clock to function
//      the 32 kHz oscillator must be connected to a 32 kHz crystal or a 32 kHz
//      clock signal must be available on pin XOSC32_Q1. The 32 kHz oscillator
//      must be started up and stable before the RTC is activated.
//      The realtime clock must be started by using the macro RTC_RUN(TRUE).
//
//  Arguments:
//      byte period
//          The desired period between interrupts in seconds.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halConfigRealTimeClock(byte period);


// Macro for stopping and starting the timers.
#define TIMER0_RUN(bool) (TR0=!!(bool))
#define TIMER1_RUN(bool) (TR1=!!(bool))
#define TIMER2_RUN(bool) (TCON2= (bool) ? TCON2|0x02 : TCON2&~0x02)
#define TIMER3_RUN(bool) (TCON2= (bool) ? TCON2|0x08 : TCON2&~0x08)
#define RTC_RUN(bool)    (RTCON= (bool) ? RTCON|0x80 : RTCON&~0x80)


// Macros pertaining to the watchdog timer.
// The watchdog must periodically be reset to avoid a system reset.
// It times out after system clock/(256*divisor)
#define WDT_DIVIDE_CLOCK_BY_2048()  ( WDT=(WDT&~0x03)|0)
#define WDT_DIVIDE_CLOCK_BY_4096()  ( WDT=(WDT&~0x03)|1)
#define WDT_DIVIDE_CLOCK_BY_8192()  ( WDT=(WDT&~0x03)|2)
#define WDT_DIVIDE_CLOCK_BY_16384() ( WDT=(WDT&~0x03)|3)
#define WDT_RESET() (WDT|=0x04)
#define WDT_ENABLE(bool)                    \
    do {                                    \
        if (bool) { WDT|=0x08; }            \
        else { WDT|=0x10; WDT&=~0x08; }     \
    } while (0)



/*****************************************************************************
 *****************************************************************************
 *************              ADC macros/functions               *************
 *****************************************************************************
 ****************************************************************************/


//----------------------------------------------------------------------------
//  bool halConfigADC(...)
//
//  Description:
//      This function configures the ADC. The _option_ argument is used to
//      select operational mode, _clkFreq_ gives the device clock frequency in 
//      kHz, and _threshold_ is used to set the threshold value for some 
//      operational modes. An interrupt is generated in all modes (except
//      for reset-generating mode in which a reset is generated instead) whenever
//      the 8 MSB of the measured sample is greater or equal to the threshold
//      value. Thus, if an interrupt for each sample is desired, the threshold
//      should be set to 0 (and the ADC and GLOBAL_ADC_DES interrupts enabled.)
//      After configuring the ADC it must be powered up using the ADC_POWER(bool)
//      macro. It should be powered down again when not in use to conserve power.
//      The correct ADC input must be selected using the ADC_SELECT_INPUT(input)
//      macro and started using the ADC_RUN(bool) macro for continuous modes.
//      The ADC_SAMPLE_SINGLE macro is used to perform a sample acquisition in
//      single-conversion mode.
//      The ADC_GET_SAMPLE_10BIT or ADC_GET_SAMPLE_8BIT macros return the latest
//      sample value.
//
//  Arguments:
//      byte options
//          One or more of the below defined constants define the desired
//          operational mode.
//      word clkFreq
//          The XOSC clock frequency in kHz.
//      byte threshold
//          The threshold value for generating interrupts (and stopping in
//          multi-conversion, stopping mode) or reset (in multi-conversion,
//          reset-generating mode).
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halConfigADC(byte options, word clkFreq, byte threshold);

// Option specifies operational mode and voltage reference source and must be
// one of the ADC_MODE_x and one of the ADC_REFERENCE_x constants below:
#define ADC_MODE_SINGLE             0x00    // Take single sample
#define ADC_MODE_MULTI_CONTINUOUS   0x10    // Continuosly take samples
#define ADC_MODE_MULTI_STOPPING     0x20    // Continuosly take samples and 
                                            // stop when value > threshold
#define ADC_MODE_MULTI_RESETTING    0x30    // Continuosly take samples and 
                                            // reset when value > threshold
#define ADC_INTERRUPT_ENABLE        0x80    // Enable ADC interrupts                                            

#define ADC_REFERENCE_VDD           0x00    // Use VDD as reference voltage
#define ADC_REFERENCE_INTERNAL_1_25 0x08    // Internal 1.25 V reference
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Macro for selecting ADC input source
#define ADC_SELECT_INPUT(input) (ADCON=(ADCON&~0x03)|(input))
// Where _input_ is one of:
#define ADC_INPUT_AD0               0
#define ADC_INPUT_AD1               1
#define ADC_INPUT_AD2               2
#define ADC_INPUT_AD0_WITH_AD1_REF  3   // Input is AD0, AD1 is used as a
                                        // reference, the ADC_REFERENCE_x
                                        // supplied to halConfigADC(...)
                                        // is ignored
//----------------------------------------------------------------------------

// Macro for powering up/down the ADC
#define ADC_POWER(bool) (ADCON=((bool) ? ADCON&~0x80 : ADCON|0x80))

// Macro for starting/stopping the ADC when in a multi-conversion mode
#define ADC_RUN(bool) (ADCON=((bool) ? ADCON|0x04 : ADCON&~0x04))

// Macro for taking a single sample in single-conversion mode
#define ADC_SAMPLE_SINGLE() do { ADCON|=0x04; while (ADCON&0x04); } while (0)

// Macro for obtaining the latest sample value (all 10 bits)
#define ADC_GET_SAMPLE_10BIT() ( (ADDATH<<8) | (ADDATL<<0) )

// Macro for obtaining the latest sample value (8 MSB)
#define ADC_GET_SAMPLE_8BIT() ( (ADDATH*64) | (ADDATL/4) )


//----------------------------------------------------------------------------
//  word halReadPotSensor(...)
//
//  Description:
//      This function reads the CC1010EB potentiometer sensor from AD0 pin.
//  Arguments:
//      void
//  Return value:
//      word
//          10 bit sample from AD0
//----------------------------------------------------------------------------
word halReadPotSensor(void);


//----------------------------------------------------------------------------
//  word halReadTempSensor(...)
//
//  Description:
//      This function reads the CC1010EM temperature sensor from AD1 pin.
//  Arguments:
//      void
//  Return value:
//      word
//          10 bit sample from AD1
//----------------------------------------------------------------------------
word halReadTempSensor(void);



/*****************************************************************************
 *****************************************************************************
 *************               DES macros/functions                *************
 *****************************************************************************
 ****************************************************************************/


//----------------------------------------------------------------------------
//  byte* halDES(...)
//
//  Description:
//      This function performs DES encryption/decryption on a block of data.
//      The encryption/decryption operations are performed in place (i.e.
//      the plaintext is overwritten by the ciphertext or vice versa) on
//      suitably aligned data [ address(_buffer_) mod 8 = 0 ]. _key_ should 
//      point to the key used for single-DES operations or keys in the case 
//      of triple-DES.
//      Two modes of the DES standard are supported: 8-bit Cipher Feedback
//      (CFB) and Output Feedback (OFB.) CFB is self-synchronizing (the end of
//      a ciphertext can be decrypted even though the beginning is unavailable)
//      and can be used to calculate error check codes. OFB is not self-
//      synchronizing and is not suitable for error check code calculation,
//      but does have the favourable property that a single bit error in the
//      received ciphertext produces only a single bit error in the decrypted
//      plaintext.
//      In CFB/OFB mode an initialization vector of 8 bytes is part of the
//      algorithm. This vector must be identical for the encryption and 
//      decryption process. The choice of initialization data does not affect
//      the security of the encryption in any way, and this function thus uses
//      the value 0.
//      _option_ is used to chose between these modes of operation, between
//      single-DES and triple-DES, and between decryption and decryption.
//      This function does not return until all data has been encrypted/
//      decrypted, since the DES hardware is so fast. The DES hardware can
//      run at the same time as the 8051 and generate an intterrupt when
//      finished. If this is desired the hardware must be programmed directly.
//
//  Arguments:
//      byte options
//          One or more of the below defined constants define the desired
//          operational mode.
//      byte xdata* key
//          A pointer to a key (or three keys for triple-DES) stored in
//          XDATA memory space. This address must be divisible by eight, i.e.
//          address(_key_) mod 8 = 0. The 56 active bits of a DES-key are
//          expected to be in a compressed 7-byte format, in which all parity
//          bits are removed. In the case of a single key, the 56 bits of the key
//          must lie on _key_[0] - _key_[6], in big-endian order. In the case of
//          three keys (triple-des), the three keys must lie on _key_[0] -
//          _key_[6], _key_[8] - _key_[14] and _key_[16] - _key_[22], all in big-
//          endian order. The macro DES_NORMAL_2_COMPACT_KEY(...) can be used
//          to convert a regular DES key to the compact form. A key can be
//          generated by simply using 7 random bytes.

//      byte xdata* buffer
//          Pointer to the data to encrypt/decrypt in XDATA memory space.
//          Encryption/decryption is performed in-place, i.e. the original
//          plaintext/ciphertext is overwritten. The address of _buffer_ must
//          be divisible by eight, i.e. address(_buffer_) mod 8 = 0.
//      word length
//          The number of bytes to perform the encryption/decryption on.
//
//  Return value:
//      byte*
//          A pointer to the start of _buffer_ is returned.
//----------------------------------------------------------------------------
byte* halDES(byte options, byte xdata* buffer, byte xdata* key, word length);

// option is one of each of:
#define DES_SINGLE_DES          0
#define DES_TRIPLE_DES          2

#define DES_ENCRYPT             0
#define DES_DECRYPT             4

#define DES_OFB_MODE            0
#define DES_CFB_MODE            8
//----------------------------------------------------------------------------


// Macro used to convert between a normal 64-bit DES-key and the
// 7-byte compressed format used by CC1010. (only useful for
// constant declarations.) Use example:
// byte my_triple_keys[]= {
//     DES_NORMAL_2_COMPACT_KEY(0xCD,0x64,0x74,0x03,0xbc,0x90,0xc4,0xc4), 0,
//     DES_NORMAL_2_COMPACT_KEY(0xDA,0xCA,0x26,0x83,0x30,0x98,0x8A,0x7D), 0,
//     DES_NORMAL_2_COMPACT_KEY(0x37,0x1E,0xC6,0x0F,0x26,0xc3,0xB2,0x9A)
// };
#define DES_NORMAL_2_COMPACT_KEY(k7, k6, k5, k4, k3, k2, k1, k0) \
        (((k7)&0xFE)<<0)|(((k6)&0x80)>>7),                       \
        (((k6)&0x7E)<<1)|(((k5)&0xC0)>>6),                       \
        (((k5)&0x3E)<<2)|(((k4)&0xE0)>>5),                       \
        (((k4)&0x1E)<<3)|(((k3)&0xF0)>>4),                       \
        (((k3)&0x0E)<<4)|(((k2)&0xF8)>>3),                       \
        (((k2)&0x06)<<5)|(((k1)&0xFC)>>2),                       \
        (((k1)&0x02)<<6)|(((k0)&0xFE)>>1)


/*****************************************************************************
 *****************************************************************************
 ************      Random number generator macros/functions      *************
 *****************************************************************************
 ****************************************************************************/

//////////////////////////////////////////////////////////////////////////////
// The CC1010 has a REAL random number generator (RNG) which uses the 140 dB
// or so of gain in the receiver path to amplify thermal noise on the shorted
// differential input of the mixer. The resulting bitstream can be used 
// directly as random numbers or used to seed a pseudo-random number 
// generator (PRNG), for instance the srand()/rand() functions in stdlib, 
// using the function below.
//////////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------------
// halRandomNumberGen(...);
//
//  Description:
//      This function activates the true RNG in the CC1010, waits long enough
//      for the output to be truly random and then samples individual random
//      bits with a period which ensures that the output is sufficiently
//      random. This function must never be used at the same time as RF is
//      in use.
//      A total of _length_ bytes of random data is stored at the location
//      pointed to by _rnd_data_.
//
//  Arguments:
//      byte* rnd_data
//          A pointer to a buffer to receive the random bytes.
//      word length
//          The number of random bytes to generate.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halRandomNumberGen(byte* rnd_data, word length);



/*****************************************************************************
 *****************************************************************************
 *************        RF communication functions/macros          *************
 *****************************************************************************
 ****************************************************************************/

//////////////////////////////////////////////////////////////////////////////
// The CC1010's on-chip radio transceiver requires that a lot of parameters
// are set and that a calibration procedure is performed before RF is used.
// The way this is abstracted in the HAL is through the notion of
// RF RX/TX Settings Pairs -- a data structure called RF_RXTXPAIR_SETTINGS
// stores all the required settings for one RX channel and one TX channel as
// a pair. This data structure can be exported from the application Chipcon 
// SmartRF Studio which allows easy configuration of such a pair.
// Each such pair to be used (normally just one, unless in a multichannel
// application) must be passed in a call to the function halRFCalib(...)
// which performs the abovementioned calibration and writes the results into
// a data structure called RF_RXTXPAIR_CALDATA.
//
// When an RX/TX-pair is to be used, the function halRFSetRxTxOff(...) should
// be called passing the corresponding RF_RXTXPAIR_SETTINGS and 
// RF_RXTXPAIR_CALDATA data structures and the desired mode (RF_RX, RF_TX, 
// RF_OFF.) This functions configures the RF parameters, powers up different
// RF modules and then finally enables the desired mode (RX/TX.) 
//
// Two functions which allow for easy reception/transmission of radio packets
// are halRFReceivePacket(...)/halRFSendPacket(...). Alternatively user-defined
// receive/send routines can be made in either a polled loop or in an 
// interrupt service routine. Several macros are defined that makes this 
// easier.
//
// The dataformat (NRZ/Manchester), baudrate (600 - 76.8k) and TX output power
// can be configured to override whatever the current RX/TX-pair is using by
// calling the functions halRFOverrideDataFormat(...), 
// halRFOverrideBaudRate(...) and halRFOverrideOutputPower(...) either before
// or after the call to halRFSetRxTxOff(...).
//
// After having received/transmitted the required data RF can be turned off
// using halRFSetRxTxOff(...) passing the parameter RF_OFF.
//
// Below is an example of how use two RX/TX-pairs in a program and how to
// transmit and send with both.
//
//
//      // RX/TX-pair data structures
//      code RF_RXTXPAIR_SETTINGS rxtxpair_settings[]={
//          { RF_RXTXPAIR_SETTINGS 0 from SmartRF Studio },
//          { RF_RXTXPAIR_SETTINGS 1 from SmartRF Studio }
//      };
//      xdata RF_RXTXPAIR_CALDATA rxtxpair_caldata[2];
//
//      // Perform calibration
//      for (i=0; i<2; i++)
//          halRFCalib(rxtxpair_settings[i], rxtxpair_caldata[i]);
//
//      // Transmit packet using pair 0
//      halRFSetRxTxOff(RF_TX, rxtxpair_settings[0], rxtxpair_caldata[0]);
//      halRFSendPacket(numPreambles, packetData, length);
//
//      // Receive packet using pair 0 and turn of RF
//      halRFSetRxTxOff(RF_RX, rxtxpair_settings[0], rxtxpair_caldata[0]);
//      halRFReceivePacket(timeOut, packetData, maxLength);
//      halRFSetRxTxOff(RF_OFF, NULL, NULL);
//
//      ...
//
//      // Transmit using pair 1 and manual handling
//      halRFSetRxTxOff(RF_TX, rxtxpair_settings[1], rxtxpair_caldata[1]);
//      RF_SEND_BYTE(RF_PREAMBLE_BYTE);         // Set the first byte to transmit
//      halRFOverrideOutputPower(RF_TX_POWER_LOW);
//      RF_START_TX();                          // Enable PA
//      for (i=0; i<num_preambles-1; i++)       // Send n preambles
//          RF_WAIT_AND_SEND_BYTE(x) 
//      RF_WAIT_AND_SEND_BYTE(RF_SUITABLE_SYNC_BYTE);   // Send sync byte
//      for (i=0; i<num_databytes; i++)         // Send n bytes of data
//          RF_WAIT_AND_SEND_BYTE(data[i]);
//      RF_WAIT_AND_SEND_BYTE(0);               // Make sure the last data byte is 
//      RF_WAIT_AND_SEND_BYTE(0);               // transmitted
//
//      // Receive using pair 1 and manual handling
//      halRFSetRxTxOff(RF_RX, rxtxpair_settings[1], rxtxpair_caldata[1]);
//      RF_SET_SYNC_BYTE(RF_SUITABLE_SYNC_BYTE);
//      RF_SET_PREAMBLE_COUNT(16);              // We want at least 16 preambles and
//      RF_START_RX();                          // then sync byte. Start RX
//      while (!RF_BYTE_RECEIVED());            // Wait for sync byte
//      INT_SETFLAG(INUM_RF, INT_CLR);          // Get ready for next byte
//      RF_LOCK_AVERAGE_FILTER(TRUE);           // Lock average filter (for NRZ)
//      for (i=0; i<num_expected_databytes; i++)
//          RF_WAIT_AND_RECEIVE_BYTE(data[i]);
//      halRFSetRxTxOff(RF_OFF);
//
//////////////////////////////////////////////////////////////////////////////


//----------------------------------------------------------------------------
// RF_RXTXPAIR_SETTINGS is a data structure which contains all the settings
// required for a given pair of RX/TX channels.
// These values should be passed to the calibration function halRFCalib(...)
// before the RX/TX channel pair can be used. The calibration function writes
// calibration results into a RF_RXTXPAIR_CALDATA data structure.
// The RX/TX channel pair is later used by passing the correpsonding
// RF_RXTXPAIR_SETTINGS and RF_RXTXPAIR_CALDATA data structures to the function
// halRFSetRxTxPair(...) and then using the function halRFSetMode(...) passing
// either RF_RX or RF_TX.
typedef struct S_RF_RXTXPAIR_SETTINGS {
    byte    modem0;             // Baudrate, xosc_freq, dataformat
    byte    modem1;             // settling, peakdetect
    byte    modem2;             // Peak level offset
    byte    freq_2a;            // RX channel frequency (A)
    byte    freq_1a;
    byte    freq_0a;
    byte    freq_2b;            // TX channel frequency (B)
    byte    freq_1b;
    byte    freq_0b;
    byte    fsep1;              // Frequency seperation
    byte    fsep0;
    byte    pll_rx;             // refdiv
    byte    pll_tx;             // refdiv
    byte    current_rx;         // vco_current, lo_drive, pa_drive
    byte    current_tx;         // vco_current, lo_drive, pa_drive
    byte    frend;              // buf_current, lna_current
    byte    pa_pow;             // TX output power
    byte    match;              // rx_match, tx_match
    byte    prescaler;          // pre_swing, pre_current
} RF_RXTXPAIR_SETTINGS;
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// RF_RXTXPAIR_CALDATA is a data structure which contains the result of a
// calibration for a given pair of RX/TX channels as specified by a
// RF_RXTXPAIR_SETTINGS data structure. (See above for details.)
typedef struct S_RF_RXTXPAIR_CALDATA {
    byte    vco_ao_rx;          // VCO array override value for RX
    byte    chp_co_rx;          // Charge-pump current override for RX
    byte    vco_ao_tx;          // VCO array override value for TX
} RF_RXTXPAIR_CALDATA;
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// Variable type used to identify result from lock process
typedef enum E_LOCK_STATUS {LOCK_NOK=0, LOCK_OK=1, LOCK_RECAL_OK=2} LOCK_STATUS;
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//  void halRFCalib(...)
//
//  Description:
//      Performs the necessary RF-calibration for an RX/TX channel pair with
//      the settings specified by the RF_RXTXPAIR_SETTINGS data structure
//      pointed to by _rf_settings_. The results of the calibration are stored
//      in the RF_RXTXPAIR_CALDATA data structure pointed to by _rf_caldata_.
//      Call this function once for each RX/TX channel pair that will be used
//      by the program before they are used and whenever the supply voltage or
//      temperature has changed significantly.
//      The application Chipcon SmartRF Studio should be used to generate this
//      data structure.
//
//  Arguments:
//      RF_RXTXPAIR_SETTINGS* rf_settings
//          Pointer to a RF_RXTXPAIR_SETTINGS data structure containing the
//          settings for an RX/TX channel pair (exported from SmartRF Studio.)
//      RF_RXTXPAIR_CALDATA xdata* rf_caldata
//          A pointer to a RF_RXTXPAIR_CALDATA data structure which will be
//          filled with the results from the calibration.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halRFCalib(
    RF_RXTXPAIR_SETTINGS code* rf_settings, 
    RF_RXTXPAIR_CALDATA xdata* rf_caldata
);


//----------------------------------------------------------------------------
//  void halRFSetRxTxOff(...)
//
//  Description:
//      Function used to set a specific RX/TX channel pair as the current RF
//      configuration. The _mode_ parameter is used to select between
//      activating RF in RX or TX or turning off RF.
//
//  Arguments:
//      byte mode
//          Options given below. Turn off, turn on in RX, or on in TX. If
//          RF_OFF is supplied the two pointers can be NULL.
//      RF_RXTXPAIR_SETTINGS* rf_settings
//          Pointer to a RF_RXTXPAIR_SETTINGS data structure containing the
//          settings for an RX/TX channel pair (exported from SmartRF Studio.)
//      RF_RXTXPAIR_CALDATA xdata* rf_caldata
//          A pointer to a RF_RXTXPAIR_CALDATA data structure which will be
//          filled with the results from the calibration.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
LOCK_STATUS halRFSetRxTxOff(
    byte mode, 
    RF_RXTXPAIR_SETTINGS code* rf_settings, 
    RF_RXTXPAIR_CALDATA xdata* rf_caldata
);

// Where _mode_ is one of
#define RF_OFF  0
#define RF_RX   1
#define RF_TX   2
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//  void halRFOverrideDataFormat(...)
//
//  Description:
//      Function used to override the dataformat specified by an RX/TX-pair.
//      If this function is called before halRFSetRxTxOff(...) it uses
//      the parameter passed to this function instead of whatever the
//      RX/TX-pair defines. If it is passed during an RF-operation, i.e. after
//      halRFSetRxTxOff(...) is called, the parameter passed is immediately
//      updated in hardware.
//
//  Arguments:
//      byte dataFormat
//          Options given below. To disable the override the function must be
//          called with the parameter RF_NO_OVERRIDE.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halRFOverrideDataFormat(byte dataFormat);

// Where _dataFormat_ is one of
#define RF_NO_OVERRIDE   0
#define RF_NRZ           0x01
#define RF_MANCHESTER    0x09
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//  void halRFOverrideBaudRate(...)
//
//  Description:
//      Function used to override the dataformat specified by an RX/TX-pair.
//      If this function is called before halRFSetRxTxOff(...) it uses
//      the parameter passed to this function instead of whatever the
//      RX/TX-pair defines. If it is passed during an RF-operation, i.e. after
//      halRFSetRxTxOff(...) is called, the parameter passed is immediately
//      updated in hardware.
//      Note that for optimal performance, the baudrate can only be changed
//      without a new calibration within the two sets (600, ..., 19200) and
//      (38400, 76800).
//
//  Arguments:
//      byte dataFormat
//          Options given below. To disable the override the function must be
//          called with the parameter RF_NO_OVERRIDE.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halRFOverrideBaudRate(byte baudRate);

// Where _dataFormat_ is one of
#define RF_NO_OVERRIDE   0
#define RF_600B          0x01
#define RF_1200B         0x21
#define RF_2400B         0x41
#define RF_4800B         0x61
#define RF_9600B         0x81
#define RF_19200B        0xA1
#define RF_38400B        0xC1
#define RF_76800B        0xE1
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//  halRFOverrideOutputPower(...)
//
//  Description:
//      Function used to override the dataformat specified by an RX/TX-pair.
//      If this function is called before halRFSetRxTxOff(...) it uses
//      the parameter passed to this function instead of whatever the
//      RX/TX-pair defines. If it is passed during an RF-operation, i.e. after
//      halRFSetRxTxOff(...) is called, the parameter passed is immediately
//      updated in hardware.
//
//  Arguments:
//      byte txPower
//          A value between 1 and 28 indicates desired power level -- 28 is
//          highest and 1 lowest. Some useful constants are defined below.
//          The override is turned off by passing 0 or RF_NO_OVERRIDE.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halRFOverrideOutputPower(byte txPower);

// Where _txPower_ is a value between 1-28 or one of
#define RF_NO_OVERRIDE      0
#define RF_TX_POWER_LOW     5
#define RF_TX_POWER_MEDIUM  18
#define RF_TX_POWER_MAX     28
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//  halRFReadRSSI(...)
//
//  Description:
//      This function activates RSSI output on the AD2 pin, activates the ADC 
//      for channel 2 reads, the RSSI output voltage and converts it to an
//      approximation of the incoming signal's strength in dBm (range is appr.
//      -110 to -50 dBm.) The value obtained is approximate and most valid for
//      a signal @ 600 MHz. The accuracy should be no worse than +/- 10 dBm at
//      any frequency.
//      An RSSI filter circuit of one capacitor and one resistor is required to
//      be connected to the AD2 pin externally (see datasheet for details).
//      This function disrupts user ADC operation and does not restore the ADC
//      to its former state.
//      The function can be used as a carrier sense if the returned value is
//      compared to some relatively low threshold.
//
//  Arguments:
//
//  Return value:
//      char
//          The approximate signal strength of the incoming signal in dBm.
//----------------------------------------------------------------------------
char halRFReadRSSI();
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//  halRFReadRSSIlevel(...)
//
//  Description:
//      This function activates RSSI output on the AD2 pin, activates the ADC 
//      for channel 2 reads, the RSSI output voltage and converts it to an
//      approximation of the incoming signal's strength in dBm (range is appr.
//      -110 to -50 dBm.) The value obtained is approximate and most valid for
//      a signal @ 600 MHz. The accuracy should be no worse than +/- 10 dBm at
//      any frequency.
//      An RSSI filter circuit of one capacitor and one resistor is required to
//      be connected to the AD2 pin externally (see datasheet for details).
//      This function disrupts user ADC operation and does not restore the ADC
//      to its former state. This function differs from the "old" function in
//      the sense that it can operate in two modes, RSSI_INIT and RSSI_RUN.
//      The ADC is not powered down after each read in order to save execution
//      time. The function can be used as a carrier sense if the returned value
//      is compared to some relatively low threshold.
//
//  Arguments:
//      byte mode
//          Selects fundtion mode, RSSI_INIT or RSSI_RUN
//
//  Return value:
//      char
//          The approximate signal strength of the incoming signal in dBm.
//----------------------------------------------------------------------------
char halRFReadRSSIlevel(byte rssi_mode);
//----------------------------------------------------------------------------

// Where _rssi_mode_ is one of
#define  RSSI_MODE_INIT 0
#define  RSSI_MODE_RUN  1

/****************************************************************************
 ************************ Manual RF Communication ***************************
 ****************************************************************************/
// If manual RF communication, i.e. without using the halRFSendPacket(...)
// and halRFReceivePacket(...) functions, is used, a few parameters must be
// set manually before before turning on RF with the macro RF_START_RX() or
// RF_START_TX() but after the call to halRFSetRxTxOff(...).
        //--------------------------------------------------------------------
        // Macro used to set the required number of preambles before the
        // synchronization byte when enabling the preamble detector. A value
        // between 0 and 127 must be set to enable the automatic preamble
        // detector. A value of 0 only looks for a sync. byte. A value greater
        // than 127 (for instance the constant RF_PREDET_OFF) disables the
        // preamble detector.
        #define RF_SET_PREAMBLE_COUNT(x)    ( PDET=(x)^0x80 )

        #define RF_PREDET_OFF 128
        //---------------------------------------------------------------------

        //---------------------------------------------------------------------
        // Macros used to set Byte or Bit RF buffering. Turns off Preamble
        // detection.
        #define RF_SET_BYTEMODE()       (PDET&=~0x80, RFCON|=0x01)
        #define RF_SET_BITMODE()        (PDET&=~0x80, RFCON&=~0x01)
        //---------------------------------------------------------------------

        //---------------------------------------------------------------------
        // Macros used to set and get the synchronization byte used during
        // preamble detection. Also a synchronization byte value suitable for
        // most  applications is defined by the constant RF_SUITABLE_SYNC_BYTE.
        #define RF_SET_SYNC_BYTE(x)     (BSYNC=(x))
        #define RF_GET_SYNC_BYTE()      BSYNC

        #define RF_SUITABLE_SYNC_BYTE   0x5A
        //---------------------------------------------------------------------

        //---------------------------------------------------------------------
        // Macro used to activate the LNA, mixer and start the modem receiving
        // data.
        #define RF_START_RX()           (EXIF&=~0x10, RFMAIN&=~0x20)
        //---------------------------------------------------------------------        
// If manual RF communication, i.e. without using the halRFSendPacket(...) and
// halRFReceivePacket(...) functions, is used, the following macros can be used to
// receive and send data bitwise or bytewise in either an interrupt service routine
// or in a polled loop.
        //---------------------------------------------------------------------
        // Macros used to send and receive individual bytes. These can
        // either be used in the RF ISR or in a loop which polls the RF
        // interrupt flag if the RF interrupt is not enabled. Either way, the
        // RF interrupt flag must be cleared between each byte/interrupt by
        // INT_SETFLAG(INUM_RF, INT_CLR). Two macros are defined to increase the
        // legibility of polled RF transfer loops (RF_READY_TO_SEND() &
        // RF_BYTE_RECEIVED() ).
        // A macro which can be used by a RF ISR to decide whether data is being
        // received or sent is RF_IS_RECEIVING().
        // Finally, a typical preamble byte is defined by the constant 
        // RF_PREAMBLE_BYTE.
        #define RF_READY_TO_SEND()          (!!(EXIF & 0x10))
        #define RF_BYTE_RECEIVED()          (!!(EXIF & 0x10))

        #define RF_SEND_BYTE(x)             (RFBUF=(x))
        #define RF_WAIT_AND_SEND_BYTE(x)            \
            do {                                    \
                while(!RF_READY_TO_SEND());         \
                RFBUF=(x);                          \
                EXIF&=~0x10;                        \
            } while (0)                        
            
        #define RF_RECEIVE_BYTE()           RFBUF
        #define RF_WAIT_AND_RECEIVE_BYTE(x)         \
            do {                                    \
                while(!RF_BYTE_RECEIVED());         \
                EXIF&=~0x10;                        \
                (x)=RFBUF;                          \
            } while (0)

        #define RF_MANCHESTER_VIOLATION     (RFCON)

        #define RF_IS_RECEIVING             (!!(RFMAIN&0x80))

        #define RF_PREAMBLE_BYTE            0xAA
        //---------------------------------------------------------------------

        //---------------------------------------------------------------------
        // Macro used to activate the power amplifier (PA) and start the modem
        // transmitting the data entered with the last RF_SEND_BYTE(x).
        #define RF_START_TX()           (EXIF&=~0x10, RFMAIN&=~0x10)
        //---------------------------------------------------------------------

        //---------------------------------------------------------------------
        // Macro which is used for locking the RX average filter, when using 
        // NRZ dataformat, upon the reception of for instance the 
        // synchronization byte.
        #define RF_LOCK_AVERAGE_FILTER(bool)  \
            ( MODEM1=(MODEM1&~0x60)|((bool) ? 0x60 : 0x20) )
        //---------------------------------------------------------------------

        //---------------------------------------------------------------------
        // Definition for lock/calibrate monitor timeout
        // Used to ensure timeout if unable to LOCK PLL within a certain time.
        #define  LOCK_TIMEOUT        0x7FFE
        #define  CAL_TIMEOUT         0x7FFE
        #define  LOCK_MONITOR_DELAY  0x0050

        //---------------------------------------------------------------------
        // Macro which is used for resetting frequency synthesizer when unable
        // to LOCK PLL. 
        #define  RF_RESET_FREQSYNTH \
                 FSCTRL &= ~0x01;   \
                 FSCTRL |= 0x01;


/****************************************************************************
 ******************** HAL Automatic RF Communication ************************
 ****************************************************************************/

//----------------------------------------------------------------------------
//  void halRFSendPacket(...)
//
//  Description:
//      Used to send a packet using the current RF configuration. (It is
//      assumed that halRFSetRxTxOff(...) has been called with the appropriate
//      RX/TX pair and mode RF_TX.) First _numPreambles_ preamble bytes
//      (RF_PREAMBLE_BYTE) are transmitted followed by a synchronization byte
//      (RF_SUITABLE_SYNC_BYTE), the _length_ byte, the data
//      pointed to by _packetData_ (_length_ bytes) and then, finally, the 
//      CRC-16 (CCITT) of the data.
//
//  Arguments:
//      byte numPreambles
//          The number of preamble bytes (RF_PREAMBLE_BYTE) to transmit.
//      byte* packetData
//          A pointer to the actual data to transmit.
//      byte length
//          The number of bytes to transmit. (Max 253)
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void halRFSendPacket(
    byte numPreambles,
    byte* packetData,
    byte length
);
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
//  void halRFReceivePacket(...)
//
//  Description:
//      Used to receive a packet sent using halSendPacket on another CC1010.
//      The current modem/RF configuration is used and RX is assumed to be
//      powered up. The function waits for a valid syncronization byte
//      (RF_SUITABLE_SYNC_BYTE) for up to _timeOut_ ms. If timed out without
//      receiving the sync. byte the function returns 0. If a sync. byte is
//      received in time the number of bytes indicated by the packet is received
//      (max _maxLength_ bytes) and put in the buffer pointed to by _packetData_,
//      after which the function returns the number of bytes received.
//      THIS FUNCTION ASSUMES THAT TIMER3 IS AVAILABLE!
//
//  Arguments:
//      byte timeOut
//          Timeout for reception of valid synchronization byte in tens of ms.
//			A value of zero gives an infinite timeout.
//      byte* packetData
//          A pointer to a buffer for the received data.
//      byte maxLength
//          The maximum number of bytes to receive (max packet size 253 bytes).
//      char* rssiByte
//          If this pointer is different from NULL an RSSI measurement is
//          performed after receiving the sync byte using the
//          halReadRSSI(...) function and its return value stored in the
//          byte pointed to by _rssiByte_.
//      word clkFreq
//          The XOSC clock frequency in kHz.
//
//  Return value:
//      byte
//          The actual number of bytes received or 0 if timed out / CRC error.
//----------------------------------------------------------------------------
byte halRFReceivePacket(
    byte  timeOut,
    byte* packetData,
    byte  maxLength,
    char* rssiByte,
    word clkFreq
);
//----------------------------------------------------------------------------



/*****************************************************************************
 *****************************************************************************
 *************                 Utility functions                 *************
 *****************************************************************************
 ****************************************************************************/

//----------------------------------------------------------------------------
//  byte* halWait(...)
//
//  Arguments:
//      byte timeOut
//          The time to wait in ms.
//      word clkFreq
//          The XOSC clock frequency in kHz.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
#if (__MODEL__ == 0)  // Memory model: 0 = SMALL, 1 = COMPACT, 2 = LARGE
void halWait(byte timeOut, word clkFreq);
#else
void halWait(byte timeOut, word clkFreq) reentrant;
#endif


#endif //HAL_H
 

