/*****************************************************************************
 *                                                                           *
 *        **********                                                         *
 *       ************                                                        *
 *      ***        ***                                                       *
 *      ***   +++   ***                                                      *
 *      ***   + +   ***                                                      *
 *      ***   +                            CHIPCON CC1010                    *
 *      ***   + +   ***              FREQUENCY JUMPING PROTOCOL              *
 *      ***   +++   ***                                                      *
 *      ***       ***                                                        *
 *       ***********                                                         *
 *        *********                                                          *
 *                                                                           *
 *****************************************************************************
 * This file contains the implementations of all the functions from the      *
 * Frequency Hopping Protocol library, which is found in Cul.h, as well as   *
 * some internal utility functions.                                          *
 *****************************************************************************
 * Author:              OGR                                                  *
 *****************************************************************************
 * Revision history:                                                         *
 *                                                                           *
 * $Log: Fhp.c,v $
 * Revision 1.1  2003/07/29 11:24:48  tos
 * Initial version in CVS.
 *
 *
 *                                                                           *
 ****************************************************************************/

#include <chipcon/hal.h>
#include <chipcon/cul.h>
#include <chipcon/cc1010eb.h>
#include <stdlib.h>

#define TRBIT 128
#define ACKBYTE 0x3B
#define PREAMBLE_BYTE_COUNT 5

#define GO_OFFLINE (FHPstatus==IDLE && FHPmode!=ACTIVE)
#define BEACON (systemID^0x80)

xdata bool isMaster = TRUE;
xdata byte FHPmode = OFF;
xdata byte FHPstatus = IDLE;

xdata byte systemID = 0x4C;
xdata byte ownID = 1;

extern xdata byte maxPacketLength;
xdata byte messageLength = 0;
xdata byte xdata* sendPacket;
xdata byte sendLength;
xdata byte sendDest;

xdata byte ctrl[2];
xdata byte requester = 0;
xdata byte errors = 99;

xdata byte lastKnownPosition = 0;

data void (*receivingFunction) () = NULL;
data void (*callback) () = NULL;
data void (*synchFailed) () = NULL;

//----------------------------------------------------------------------------
//  void showLED(...)
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Displays the four least significant bits of the argument
//      on the LEDs on the evaluation board.
//      Used for debugging purposes.
//
//  Arguments:
//      byte a
//          The value to be displayed on the LEDs.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void showLED(byte a) {
	BLED = !(a&0x01);
	GLED = !(a&0x02);
	YLED = !(a&0x04);
	RLED = !(a&0x08);
} // end showLED

//----------------------------------------------------------------------------
//  void fhpSendPacket()
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Transmits a packet, possibly receives acknowledgement
//      and calls the callback function if successfull.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void fhpSendPacket() {
	bool success = TRUE;
	halRFSendPacket(PREAMBLE_BYTE_COUNT, &sendPacket[0], sendLength);
	if (fhpSettings.useAck && (sendDest!=0 || !isMaster)) {
		changeFreq(RF_RX, 0);
		halWait(1,fhpSettings.clkFreq);
		if (!(halRFReceivePacket(fhpSettings.listenTime, &ctrl[0], 2, 0, fhpSettings.clkFreq)==2 && ctrl[0]==systemID && ctrl[1]==ACKBYTE)) {
			success = FALSE;
		}
	}
	if (success) {
		FHPstatus = IDLE;
		if (callback != NULL)
			callback();
	}
} // end fhpSendPacket

//----------------------------------------------------------------------------
//  void fhpReceivePacket(...)
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Receives a packet and possibly sends an acknowledgement.
//      The receivingFunction is called if successfull.
//
//  Arguments:
//      bool sendAck
//          Specifies whether or not to send an acknowledgement.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void fhpReceivePacket(bool sendAck) {
	messageLength = halRFReceivePacket(fhpSettings.listenTime, message, maxPacketLength, 0, fhpSettings.clkFreq);
	if (messageLength > 0) {
		if (fhpSettings.useAck && sendAck) {
			changeFreq(RF_TX, 0);
			ctrl[0] = systemID;
			ctrl[1] = ACKBYTE;
			halWait(1,fhpSettings.clkFreq);
			halRFSendPacket(PREAMBLE_BYTE_COUNT, &ctrl[0], 2);
		}
		if (receivingFunction!=NULL)
			receivingFunction();
	}
} // end fhpReceivePacket

//----------------------------------------------------------------------------
//  void setupTimer2(word period)
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Sets up timer 2 as an interrupt timer and enables it.
//
//  Arguments:
//      word period
//          The value to be loaded into the timer whenever it overflows.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void setupTimer2(word period) {
    EXIF&=~0x80;    // Clear interrupt flag
    ET2=1;          // Enable timer interrupt
    EA=1;           // Enable global interrupt
    T2PRE=(byte)(period>>8);
    T2=(byte)period;
    TCON2=(TCON2&~0x0C);
    INT_PRIORITY(INUM_TIMER2, INT_HIGH);
	TIMER2_RUN(FALSE);
} // end setupTimer2

//----------------------------------------------------------------------------
//  void initializeFHP(...)
//
//  Description:
//      Used to initialize the FHP protocol as either a master or a slave.
//      Frequencies are calibrated and timers are set up.
//
//  Arguments:
//      bool master
//          TRUE if master, FALSE if slave
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void initializeFHP(bool master) {
	isMaster = master;
	if (fhpSettings.useLEDs) showLED(15);
	initializeFrequencies();
	if (fhpSettings.useLEDs) showLED(0);
	if (master) {
		setupTimer2((word)(((ulong)fhpSettings.period*fhpSettings.clkFreq+127)/255));
	} else {
		setModulus(fhpSettings.period*(ulong)1000*numberOfFrequencies);
		setupTimer2((word)((((ulong)(fhpSettings.period-fhpSettings.overhead))*fhpSettings.clkFreq+127)/255)); //-5*fhpSettings.listenTime
	}
} // end initializeFHP



//----------------------------------------------------------------------------
//  void setReceivingFunction(...)
//
//  Description:
//      Used to specify which function should be called when
//      a packet arrives. The function may simply set a flag to
//      indicate the arrival, or it may process the packet itself.
//      The packet is available in the buffer message until a new
//      packet arrives.
//      An argument of NULL specifies that no function will be called.
//      This is the default behaviour on start-up.
//
//  Arguments:
//      void (*pF) ()
//          A pointer to the function that will be notified of new packets.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setReceivingFunction( void (*pF) () ) {
	receivingFunction = pF;
} // end setReceivingFunction



//----------------------------------------------------------------------------
//  void setCallbackFunction(...)
//
//  Description:
//      Used to specify which function should be called when
//      a packet is successfully sent.
//      An argument of NULL specifies that no function will be called.
//      This is the default behaviour on start-up.
//
//  Arguments:
//      void (*pF) ()
//          A pointer to the function that will be notified
//          of successful transmissions.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setCallbackFunction( void (*pF) () ) {
	callback = pF;
} // end setCallbackFunction



//----------------------------------------------------------------------------
//  void setSynchronizationFailedFunction(...)
//
//  Description:
//      Used to specify which function should be called when
//      an acquisition attempt is unsuccessful.
//      The user application must then initiate a new attempt.
//      This could be done in a number of ways:
//        - calling the synchronize function
//        - calling setMode(ACTIVE)
//        - sending a new packet if in the PASSIVE mode
//          (any previous packet must be canceled)
//      An argument of NULL specifies that the FHP should go on trying
//      to synchronize until it is successful instead of calling a
//      user function. This is the default behaviour on start-up.
//
//  Arguments:
//      void (*pF) ()
//          A pointer to the function that will be notified failed acquisitions.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setSynchronizationFailedFunction( void (*pF) () ) {
	synchFailed = pF;
} // end setSynchronizationFailedFunction

//----------------------------------------------------------------------------
//  void resetTimers()
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Resets timer 2 and the real time clock.
//      To be called when contact is established with the master.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void resetTimers() {
	TIMER2_RUN(FALSE);
	TIMER2_RUN(TRUE);
	setTime(1000*(ulong)(fhpSettings.overhead+fhpSettings.period/2));
	lastKnownPosition = getFrequencyPosition();
	errors = 0;
} // end resetTimers

//----------------------------------------------------------------------------
//  bool listenForBeacon(...)
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Receives the beacon transmitted by the master and possibly
//      receives a packet, transmits a packet or transmits a signal
//      to indicate that the slave wants to transmit a packet.
//
//  Arguments:
//      byte timeout
//          The time in tenths of milliseconds that it will listen for beacon.
//  Return value:
//      bool
//          Indicates whether or not a beacon was received.
//----------------------------------------------------------------------------
bool listenForBeacon(byte timeout) {
	byte length, control;
	if (fhpSettings.useLEDs) RLED = GLED = LED_OFF;
	length = halRFReceivePacket(timeout, &ctrl[0], 2, 0, fhpSettings.clkFreq);
	if (fhpSettings.useLEDs) YLED = LED_OFF;
	if (length==2 && ctrl[0]==BEACON) {
		resetTimers();
		if (FHPstatus == NOLINK)
			FHPstatus = WAITING;
		if (fhpSettings.useLEDs) {
			GLED = LED_ON;
			RLED = LED_OFF;
        }
		control = ctrl[1];
		// Respond
		if (control&TRBIT) {
			// Master is going to send
			if ( (control==(BROADCASTID|TRBIT)) || (control==(ownID|TRBIT)) ) {
				// The packet will be for me
				fhpReceivePacket(control==(ownID|TRBIT));
			}
		} else if ( FHPstatus != IDLE ) {
			// A packet is ready
			if (control == BROADCASTID) {
				// nobody will send
				changeFreq(RF_TX, 0);
				ctrl[0] = systemID;
				ctrl[1] = ownID;
				halWait((rand()%fhpSettings.timeslots)*fhpSettings.slotSize,fhpSettings.clkFreq);
				halRFSendPacket(PREAMBLE_BYTE_COUNT, &ctrl[0], 2);
			} else if (control == ownID) {
				changeFreq(RF_TX, 0);
				fhpSendPacket();
				if (GO_OFFLINE)
					TIMER2_RUN(FALSE);
			} // else someone else will send
		}
		return TRUE;
	} else {
		++errors;
		if (fhpSettings.useLEDs) RLED = LED_ON;
		return FALSE;
	}
} // end listenForBeacon



//----------------------------------------------------------------------------
//  bool newSending(...)
//
//  Description:
//      Initiates a new transmission.
//      The previous transmission must either have been successfully completed
//      or canceled.
//      The function returns immediately, and the packet must not be altered
//      before the packet has been sent or canceled.
//
//  Arguments:
//      byte xdata* packet
//          A pointer to the packet, which must reside in external memory.
//      byte length
//          The length of the packet in bytes.
//      byte destination
//          If the FHP is set up as slave, this argument has no effect as the
//          packet is sent to the master anyway.
//          If set up as master, the argument specifies the ID of the slave
//          to receive the packet or BROADCASTID if all slaves should receive
//          the packet.
//          
//  Return value:
//      bool
//          Returns TRUE if the previous transmission either has been
//          successfully completed or canceled. Otherwise, FALSE is returned
//          and the new packet is discarded.
//----------------------------------------------------------------------------
bool newSending(byte xdata* packet, byte length, byte destination) {
	if (FHPstatus != IDLE)
		return FALSE;
	sendLength = length;
	sendDest = destination;
	sendPacket = packet;
	FHPstatus = WAITING;
	if (FHPmode != ACTIVE) {
		errors = 99;
		TIMER2_RUN(TRUE);
	}
	return TRUE;
} // end newSending



//----------------------------------------------------------------------------
//  byte getSendStatus()
//
//  Description:
//      Used to check the status of the last initiated transmission.
//
//  Arguments:
//      None.
//          
//  Return value:
//      byte
//          The return value is one of:
//           - IDLE if the last transmission is canceled or completed
//           - NOLINK if this is a slave and
//             there is no radio link with master
//           - WAITING otherwise
//----------------------------------------------------------------------------
byte getSendStatus() {
	return FHPstatus;
} // end getSendStatus



//----------------------------------------------------------------------------
//  void cancelSending()
//
//  Description:
//      Cancels the current transmission. Thus, it will not be transmitted
//      and a new transmission may be initiated.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void cancelSending() {
	FHPstatus = IDLE;
} // end cancelSending



//----------------------------------------------------------------------------
//  bool synchronize()
//
//  Description:
//      Performs an acquisition by scanning through the frequency sequence.
//
//  Arguments:
//      None.
//          
//  Return value:
//      bool
//          TRUE if a stable connection was obtained, FALSE otherwise
//----------------------------------------------------------------------------
bool synchronize() {
	byte i,hits=0,ptr0,ptr1;
	bool turn = FALSE;
	ulong time = getTime()/1000;
	byte step = time/fhpSettings.period+1;
	word wait = (word)(fhpSettings.period*(ulong)step-time);
//	ET2 = 0; // disable timer 2
	while (wait>255) {
		halWait(255, fhpSettings.clkFreq);
		wait-=255;
	}
	halWait(wait, fhpSettings.clkFreq);
	if (fhpSettings.useLEDs) BLED = LED_ON;
	ptr0 = lastKnownPosition+step;
	ptr1 = lastKnownPosition+step+2;
	for (i=1; i<=numberOfFrequencies; ++i) {
		if (hits) {
			changeFreq(RF_RX, 1);
			++ptr0;
			++ptr1;
			--i;
		} else {
//#define SIMPLE_STEPPING
#ifdef SIMPLE_STEPPING
			changeFreq(RF_RX, 2);
#else
			if (turn) {
				if (ptr1>=numberOfFrequencies)
					ptr1-=numberOfFrequencies;
				setFrequencyPosition(ptr1);
				ptr1 += 3;
			} else {
				if (ptr0>=numberOfFrequencies)
					ptr0-=numberOfFrequencies;
				setFrequencyPosition(ptr0++);
			}
			changeFreq(RF_RX,0);
			turn = !turn;
#endif
		}
		if (listenForBeacon(fhpSettings.period/10)) {
			++hits;
			// require two hits in a row to avoid being fooled by interchannel noise
			if (hits>=2) { //|| (GO_OFFLINE && !force)
				changeFreq(RF_OFF, 1);
				if (fhpSettings.useLEDs) BLED = LED_OFF;
				ET2 = 1;
				return TRUE;
			}
		} else {
			hits = 0;
		}
	}
	changeFreq(RF_OFF,0);
//	ET2 = 1;
	return FALSE;
} // end synchronize


//----------------------------------------------------------------------------
//  void setMode(...)
//
//  Description:
//      Determines how active the FHP will be.
//      If the FHP is set up as master, there are two alternatives:
//       - OFF:     The FHP is turned off and does nothing
//       - ACTIVE:  Beacons are transmitted periodically,
//                  packets from the slaves are received and
//                  any packets sent with newSending are transmitted.
//      If the FHP is set up as slave, there are three alternatives:
//       - OFF:     The FHP is turned off and does nothing
//       - PASSIVE: A timer is running in the background keeping track of
//                  where in the frequency sequence the master is.
//                  Packets sent with newSending are transmitted fast,
//                  but nothing is received (unless possibly when
//                  transmitting a packet itself).
//       - ACTIVE:  The FHP listens for every beacon transmitted by
//                  the master. Thus, the communication is full duplex.
//
//  Arguments:
//      byte mode
//          One of OFF, PASSIVE or ACTIVE
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setMode(byte mode) {
	byte oldMode = FHPmode;
	if (isMaster && mode==PASSIVE)
		mode = OFF; // Master has no PASSIVE mode
	FHPmode = mode;
	switch (mode) {
		case OFF:
			RTstop();
			break;
		case PASSIVE:
			if (oldMode == OFF) {
				RTstart();
			}
			if (FHPstatus == IDLE)
				TIMER2_RUN(FALSE);
			break;
		case ACTIVE:
			if (!isMaster) {
				if (oldMode==OFF) {
					RTstart();
				}
			}
			errors = 99;
			TIMER2_RUN(TRUE);
			break;
		default:
			return; // error: undefined mode
	}
} // end setMode



//----------------------------------------------------------------------------
//  void setSystemID(...)
//
//  Description:
//      Sets the system ID. The master and all slaves that are to communicate
//      with each other must have the same system ID. The reason is that
//      all beacons and acknowledgements contain the system ID and are
//      discarded if they do not match. Thus, giving systems different
//      IDs prevents them from interfering with each other.
//      All application programs should choose a system ID different from
//      the default one, preferably chosen randomly for each particular system.
//
//  Arguments:
//      byte ID
//          The new system ID. Range 0-127.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void setSystemID(byte ID) {
	systemID = ID;
} // end setSystemID




//----------------------------------------------------------------------------
//  void setOwnID(...)
//
//  Description:
//      If the FHP is set up as master, this function has no effect.
//      Otherwise, it specifies the ID of the slave. The slave will only
//      receive packets addressed to the BROADCASTID or to its own ID.
//      The most significant bit in the ID must be 0, and the ID must not
//      equal the BROADCASTID (0).
//
//  Arguments:
//      byte ID
//          The new ID of the slave. Must be in the range 1-127.
//
//  Return value:
//      void
//----------------------------------------------------------------------------
void setOwnID(byte ID) {
	srand(ID);
	ownID = ID&~TRBIT;
} // end setOwnID



//----------------------------------------------------------------------------
//  void fhpSlave()
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Called periodically to listen for the beacons transmitted by the master.
//      If the beacon is missed several times in a row, an acquisition is performed.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void fhpSlave() {
	if (errors >= 3) {
		if (FHPstatus == WAITING)
			FHPstatus = NOLINK;
		while (!synchronize()) {
			if (synchFailed!=NULL) {
				TIMER2_RUN(FALSE);
				synchFailed();
				break;
			}
		}
	} else {
		changeFreq(RF_RX,0);
		listenForBeacon(fhpSettings.listenTime);
		changeFreq(RF_OFF, 1);
	}
} // end slave



//----------------------------------------------------------------------------
//  void fhpMaster()
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Called periodically to send out beacons and any packets,
//      and possibly receives packets from the slaves.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void fhpMaster() {
	byte length;
	// Send beacon
	changeFreq(RF_TX,0);
	ctrl[0] = BEACON;
	if (FHPstatus != IDLE) {
		ctrl[1] = sendDest | TRBIT;
	} else if (requester) {
		ctrl[1] = requester;
	} else {
		ctrl[1] = BROADCASTID;
	}
    halRFSendPacket(PREAMBLE_BYTE_COUNT, &ctrl[0], 2);
	if (ctrl[1]&TRBIT) {
		// I will send something myself
		fhpSendPacket();
	} else if (requester) {
		changeFreq(RF_RX, 0);
		fhpReceivePacket(TRUE);
		requester = 0;
	} else {
		changeFreq(RF_RX, 0);
		length = halRFReceivePacket((fhpSettings.timeslots*fhpSettings.slotSize+9)/10, &ctrl[0], 2, 0, fhpSettings.clkFreq);
		if (length==2 && ctrl[0]==systemID) {
			requester = ctrl[1];
			if (fhpSettings.useLEDs) GLED = LED_ON;
		} else {
			if (fhpSettings.useLEDs) RLED = LED_ON;
		}
	}
	changeFreq(RF_OFF, 1);
	if (fhpSettings.useLEDs) YLED = LED_OFF;
} //end master



//----------------------------------------------------------------------------
//  void fhpJump() interrupt INUM_TIMER2
//
//  Description:
//      Internal function only to be used by the FHP itself.
//      Interrupt service routine called periodically when in ACTIVE mode.
//      Calls fhpMaster() or fhpSlave() depending on configuration.
//
//  Arguments:
//      None.
//          
//  Return value:
//      void
//----------------------------------------------------------------------------
void fhpJump() interrupt INUM_TIMER2 {

	INT_SETFLAG(INUM_TIMER2, INT_CLR);

   	if ( GO_OFFLINE ) {
		TIMER2_RUN(FALSE);
		return;
	}

	if (fhpSettings.useLEDs) {
		GLED = LED_OFF;
		RLED = LED_OFF;
		YLED = LED_ON;
	}

	if (isMaster)
		fhpMaster();
	else
		fhpSlave();

	INT_SETFLAG(INUM_TIMER2, INT_CLR);
} // end jump

